from __future__ import absolute_import

from .models import glyphs, markers, BoxSelectTool

def _glyph_function(glyphclass, dsnames, argnames, docstring):

    def func(plot, *args, **kwargs):

        # Note: We want to reuse the glyph functions by attaching them the Plot
        # class. Imports are here to prevent circular imports.
        from .plotting_helpers import (
            _match_args, _pop_renderer_args, _pop_colors_and_alpha, _process_sequence_literals,
            _update_legend, _make_glyph)
        from .models import GlyphRenderer, Plot

        if not isinstance(plot, Plot):
            raise ValueError("expected plot object for first argument")

        # pop off glyph *function* parameters that are not glyph class properties
        legend_name = kwargs.pop("legend", None)
        renderer_kws = _pop_renderer_args(kwargs)
        source = renderer_kws['data_source']

        # pop off all color values for the glyph or nonselection glyphs
        glyph_ca = _pop_colors_and_alpha(glyphclass, kwargs)
        nsglyph_ca = _pop_colors_and_alpha(glyphclass, kwargs, prefix='nonselection_', default_alpha=0.1)

        # add the positional arguments as kwargs and make sure all required args are present
        _match_args(dsnames, glyphclass, source, args, kwargs)

        # if there are any hardcoded data sequences, move them to the data source and update
        _process_sequence_literals(glyphclass, kwargs, source)
        _process_sequence_literals(glyphclass, glyph_ca, source)
        _process_sequence_literals(glyphclass, nsglyph_ca, source)

        # create the default and nonselection glyphs
        glyph = _make_glyph(glyphclass, kwargs, glyph_ca)
        nsglyph = _make_glyph(glyphclass, kwargs, nsglyph_ca)

        glyph_renderer = GlyphRenderer(glyph=glyph, nonselection_glyph=nsglyph, **renderer_kws)

        if legend_name:
            _update_legend(plot, legend_name, glyph_renderer)

        for tool in plot.select(type=BoxSelectTool):
            tool.renderers.append(glyph_renderer)
            tool._dirty = True

        plot.renderers.append(glyph_renderer)
        plot._dirty = True
        return glyph_renderer

    func.__name__ = glyphclass.__view_model__
    func.__doc__ = docstring
    return func

annular_wedge = _glyph_function(glyphs.AnnularWedge, ("x", "y", "inner_radius", "outer_radius", "start_angle", "end_angle"), ("direction",),
""" Add :class:`~bokeh.models.glyphs.AnnularWedge` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    inner_radius (str or list[float]) : values or field names of inner radii
    outer_radius (str or list[float]) : values or field names of outer radii
    start_angle (str or list[float]) : values or field names of starting angles
    end_angle (str or list[float]) : values or field names of ending angles
    direction ("clock" or "anticlock", optional): direction to turn between starting and ending angles, defaults to "anticlock"
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

"""
)

annulus = _glyph_function(glyphs.Annulus, ("x", "y" ,"inner_radius", "outer_radius"), (),
""" Add :class:`~bokeh.models.glyphs.Annulus` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    inner_radius (str or list[float]) : values or field names of inner radii
    outer_radius (str or list[float]) : values or field names of outer radii
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.annulus(x=[1, 2, 3], y=[1, 2, 3], color="#7FC97F",
                     inner_radius=0.2, outer_radius=0.5)

        show(plot)

"""
)

arc = _glyph_function(glyphs.Arc, ("x", "y", "radius" ,"start_angle", "end_angle"), ("direction",),
""" Add :class:`~bokeh.models.glyphs.Arc` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    radius (str or list[float]) : values or field names of arc radii
    start_angle (str or list[float]) : values or field names of starting angles
    end_angle (str or list[float]) : values or field names of ending angles
    direction ("clock" or "anticlock", optional): direction to turn between starting and ending angles, defaults to "anticlock"
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

"""
)

asterisk = _glyph_function(markers.Asterisk, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Asterisk` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.asterisk(x=[1,2,3], y=[1,2,3], size=20, color="#F0027F")

        show(plot)

"""
)

bezier = _glyph_function(glyphs.Bezier, ("x0", "y0", "x1", "y1", "cx0", "cy0", "cx1", "cy1"), (),
""" Add :class:`~bokeh.models.glyphs.Bezier` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x0 (str or list[float]) : values or field names of starting x coordinates
    y0 (str or list[float]) : values or field names of starting y coordinates
    x1 (str or list[float]) : values or field names of ending x coordinates
    y1 (str or list[float]) : values or field names of ending y coordinates
    cx0 (str or list[float]) : values or field names of first control point x coordinates
    cy0 (str or list[float]) : values or field names of first control point y coordinates
    cx1 (str or list[float]) : values or field names of second control point x coordinates
    cy1 (str or list[float]) : values or field names of second control point y coordinates
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

""")

circle = _glyph_function(markers.Circle, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Circle` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float], optional) : values or field names of sizes in screen units
    radius (str  or list[float], optional): values or field names of radii
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

.. note::
    Only one of ``size`` or `radius` should be provided. Note that `radius` defaults to data units.

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.circle(x=[1, 2, 3], y=[1, 2, 3], size=20)

        show(plot)

"""
)

circle_cross = _glyph_function(markers.CircleCross, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.CircleCross` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.circle_cross(x=[1,2,3], y=[4,5,6], size=20,
                          color="#FB8072", fill_alpha=0.2, line_width=2)

        show(plot)

"""
)

circle_x = _glyph_function(markers.CircleX, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.CircleX` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.circle_x(x=[1, 2, 3], y=[1, 2, 3], size=20,
                     color="#DD1C77", fill_alpha-0.2)

        show(plot)

"""
)

cross = _glyph_function(markers.Cross, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Cross` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.cross(x=[1, 2, 3], y=[1, 2, 3], size=20,
                   color="#E6550D", line_width=2)

        show(plot)

"""
)

diamond = _glyph_function(markers.Diamond, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Diamond` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.diamond(x=[1, 2, 3], y=[1, 2, 3], size=20,
                    color="#1C9099", line_width=2)

        show(plot)

"""
)

diamond_cross = _glyph_function(markers.DiamondCross, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.DiamondCross` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.diamond_cross(x=[1, 2, 3], y=[1, 2, 3], size=20,
                           color="#386CB0", fill_color=None, line_width=2)

        show(plot)

"""
)

image = _glyph_function(glyphs.Image, ("image", "x", "y", "dw", "dh"), ('palette', 'reserve_color', 'reserve_val', 'color_mapper', 'dilate'),
""" Add :class:`~bokeh.models.glyphs.Image` glyphs to a :class:`~bokeh.plotting.Figure`.

A palette (string name of a built-in palette, currently) must also be supplied to use for
color-mapping the scalar image.

Args:
    image (str or 2D array_like of float) : value or field names of scalar image data
    x (str or list[float]) : values or field names of lower left x coordinates
    y (str or list[float]) : values or field names of lower left y coordinates
    dw (str or list[float]) : values or field names of image width distances
    dh (str or list[float]) : values or field names of image height distances
    palette (str or list[str]) : values or field names of palettes to use for color-mapping (see :ref:`bokeh_dot_palettes` for more details)
    color_mapper (LinearColorMapper) : a LinearColorMapper instance
    dilate (bool, optional) : whether to dilate pixel distance computations when drawing, defaults to False

Returns:
    GlyphRenderer

.. note::
    Setting `dilate` to True will cause pixel distances (e.g., for `dw` and `dh`) to
    be rounded up, always.

"""
)

image_rgba = _glyph_function(glyphs.ImageRGBA, ("image", "x", "y", "dw", "dh"), ("dilate",),
""" Add :class:`~bokeh.models.glyphs.ImageRGBA` glyphs to a :class:`~bokeh.plotting.Figure`.

The ``image_rgba`` method accepts images as a two-dimensional array of RGBA values (encoded
as 32-bit integers).

Args:
    image (str or 2D array_like of uint32) : value or field names of RGBA image data
    x (str or list[float]) : values or field names of lower left x coordinates
    y (str or list[float]) : values or field names of lower left y coordinates
    dw (str or list[float]) : values or field names of image width distances
    dh (str or list[float]) : values or field names of image height distances
    dilate (bool, optional) : whether to dilate pixel distance computations when drawing, defaults to False

Returns:
    GlyphRenderer

.. note::
    Setting ``dilate`` to True will cause pixel distances (e.g., for ``dw`` and ``dh``) to
    be rounded up, always.

"""
)

image_url = _glyph_function(glyphs.ImageURL, ("url", "x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.ImageURL` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    url (str) : value of RGBA image data
    x (str or list[float]) : values or field names of upper left x coordinates
    y (str or list[float]) : values or field names of upper left y coordinates
    angle (float) : angle to rotate image by

Returns:
    GlyphRenderer

"""
)

inverted_triangle = _glyph_function(markers.InvertedTriangle, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.InvertedTriangle` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer


Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.inverted_triangle(x=[1, 2, 3], y=[1, 2, 3], size=20, color="#DE2D26")

        show(plot)

"""
)

line = _glyph_function(glyphs.Line, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Line` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of line x coordinates
    y (str or list[float]) : values or field names of line y coordinates
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
       :source-position: above

       from bokeh.plotting import figure, output_file, show

       p = figure(title="line", plot_width=300, plot_height=300)
       p.line(x=[1, 2, 3, 4, 5], y=[6, 7, 2, 4, 5])

       show(p)

"""
)

multi_line = _glyph_function(glyphs.MultiLine, ("xs", "ys"), (),
""" Add :class:`~bokeh.models.glyphs.MultiLine` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    xs (str or list[list[float]]): values or field names of lines x coordinates
    ys (str or list[list[float]]): values or field names of lines y coordinates
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

.. note::
    For this glyph, the data is not simply an array of scalars, it is really
    an "array of arrays".

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
       :source-position: above

       from bokeh.plotting import figure, output_file, show

       p = figure(plot_width=300, plot_height=300)
       p.multi_line(xs=[[1, 2, 3], [2, 3, 4]], ys=[[6, 7, 2], [4, 5, 7]],
                    color=['red','green'])

       show(p)

""")

oval = _glyph_function(glyphs.Oval, ("x", "y", "width", "height"), (),
""" Add :class:`~bokeh.models.glyphs.Oval` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    width (str or list[float]) : values or field names of widths
    height (str or list[float]) : values or field names of heights
    angle (str or list[float], optional) : values or field names of rotation angles, defaults to 0
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.oval(x=[1, 2, 3], y=[1, 2, 3], width=0.2, height=0.4,
                  angle=-0.7, color="#1D91C0")

        show(plot)

"""
)

patch = _glyph_function(glyphs.Patch, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Patch` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of patch x coordinates
    y (str or list[float]) : values or field names of patch y coordinates
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
       :source-position: above

       from bokeh.plotting import figure, output_file, show

       p = figure(plot_width=300, plot_height=300)
       p.patch(x=[1, 2, 3, 2], y=[6, 7, 2, 2], color="#99d8c9")

       show(p)

"""
)

patches = _glyph_function(glyphs.Patches, ("xs", "ys"), (),
""" Add :class:`~bokeh.models.glyphs.Patches` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    xs (str or list[list[float]]): values or field names of patches x coordinates
    ys (str or list[list[float]]): values or field names of patches y coordinates
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

.. note::
    For this glyph, the data is not simply an array of scalars, it is really
    an "array of arrays".

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
       :source-position: above

       from bokeh.plotting import figure, output_file, show

       p = figure(plot_width=300, plot_height=300)
       p.patches(xs=[[1,2,3],[4,5,6,5]], ys=[[1,2,1],[4,5,5,4]],
                color=["#43a2ca", "#a8ddb5"])

       show(p)

""")

quad = _glyph_function(glyphs.Quad, ("left", "right", "top", "bottom"), (),
""" Add :class:`~bokeh.models.glyphs.Quad` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    left (str or list[float]) : values or field names of left edges
    right (str or list[float]) : values or field names of right edges
    top (str or list[float]) : values or field names of top edges
    bottom (str or list[float]) : values or field names of bottom edges
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.quad(top=[2, 3, 4], bottom=[1, 2, 3], left=[1, 2, 3],
            right=[1.2, 2.5, 3.7], color="#B3DE69")

        show(plot)

""")

quadratic = _glyph_function(glyphs.Quadratic, ("x0", "y0", "x1", "y1", "cx", "cy"), (),
""" Add :class:`~bokeh.models.glyphs.Quadratic` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x0 (str or list[float]) : values or field names of starting x coordinates
    y0 (str or list[float]) : values or field names of starting y coordinates
    x1 (str or list[float]) : values or field names of ending x coordinates
    y1 (str or list[float]) : values or field names of ending y coordinates
    cx (str or list[float]) : values or field names of control point x coordinates
    cy (str or list[float]) : values or field names of control point y coordinates
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

""")

ray = _glyph_function(glyphs.Ray, ("x", "y", "length", "angle"), (),
""" Add :class:`~bokeh.models.glyphs.Ray` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    length (str or list[float]) : values or field names of ray lengths in screen units
    angle (str or list[float]) : values or field names of ray angles
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.ray(x=[1, 2, 3], y=[1, 2, 3], length=45, angle=-0.7, color="#FB8072",
                 line_width=2)

        show(plot)

"""
)

rect = _glyph_function(glyphs.Rect, ("x", "y", "width", "height"), ("dilate",),
""" Add :class:`~bokeh.models.glyphs.Rect` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    width (str or list[float]) : values or field names of widths
    height (str or list[float]) : values or field names of heights
    angle (str or list[float], optional) : values or field names of rotation angles, defaults to 0
    dilate (bool, optional) : whether to dilate pixel distance computations when drawing, defaults to False
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

.. note::
    Setting `dilate` to True will cause pixel distances (e.g., for `width` and `height`) to
    be rounded up, always.


Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.rect(x=[1, 2, 3], y=[1, 2, 3], width=10, height=20, color="#CAB2D6",
            width_units="screen", height_units="screen")

        show(plot)

"""
)

segment = _glyph_function(glyphs.Segment, ("x0", "y0", "x1", "y1"), (),
""" The ``segment`` glyph displays line segments with the given starting and ending coordinates.

Args:
    x0 (str or list[float]) : values or field names of starting x coordinates
    y0 (str or list[float]) : values or field names of starting y coordinates
    x1 (str or list[float]) : values or field names of ending x coordinates
    y1 (str or list[float]) : values or field names of ending y coordinates
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.segment(x0=[1, 2, 3], y0=[1, 2, 3], x1=[1, 2, 3],
                    y1=[1.2, 2.5, 3.7], color="#F4A582",
                    line_width=3)

        show(plot)

""")

square = _glyph_function(markers.Square, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Square` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.square(x=[1, 2, 3], y=[1, 2, 3], size=[10,20,30], color="#74ADD1")

        show(plot)

"""
)

square_cross = _glyph_function(markers.SquareCross, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.SquareCross` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.square_cross(x=[1, 2, 3], y=[1, 2, 3], size=[10,20,25],
                         color="#7FC97F",fill_color=None, line_width=2)

        show(plot)

"""
)

square_x = _glyph_function(markers.SquareX, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.SquareX` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.square_x(x=[1, 2, 3], y=[1, 2, 3], size=[10,20,25],
                     color="#FDAE6B",fill_color=None, line_width=2)

        show(plot)

"""
)

text = _glyph_function(glyphs.Text, ("x", "y", "text"), (),
""" Add :class:`~bokeh.models.glyphs.Text` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of text x coordinates
    y (str or list[float]) : values or field names of text y coordinates
    text (str or list[text]): values or field names of texts
    angle (str or list[float], optional) : values or field names of text angles, defaults to 0
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_text_properties`

.. note::
    The location and angle of the text relative to the ``x``, ``y`` coordinates
    is indicated by the alignment and baseline text properties.

Returns:
    GlyphRenderer

"""
)

triangle = _glyph_function(markers.Triangle, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.Triangle` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.triangle(x=[1, 2, 3], y=[1, 2, 3], size=[10,20,25],
                     color="#99D594", line_width=2)

        show(plot)

"""
)

wedge = _glyph_function(glyphs.Wedge, ("x", "y", "radius", "start_angle", "end_angle"), ("direction",),
""" Add :class:`~bokeh.models.glyphs.Wedge` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    radius (str or list[float]) : values or field names of wedge radii
    start_angle (str or list[float]) : values or field names of starting angles
    end_angle (str or list[float]) : values or field names of ending angles
    direction ("clock" or "anticlock", optional): direction to turn between starting and ending angles, defaults to "anticlock"
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.wedge(x=[1, 2, 3], y=[1, 2, 3], radius=15, start_angle=0.6,
                     end_angle=4.1, radius_units="screen", color="#2b8cbe")

        show(plot)

"""
)

x = _glyph_function(markers.X, ("x", "y"), (),
""" Add :class:`~bokeh.models.glyphs.X` glyphs to a :class:`~bokeh.plotting.Figure`.

Args:
    x (str or list[float]) : values or field names of center x coordinates
    y (str or list[float]) : values or field names of center y coordinates
    size (str or list[float]) : values or field names of sizes in screen units
    source (:class:`~bokeh.models.sources.ColumnDataSource`, optional) : a user-supplied data source.
        If none is supplied, one is created for the user automatically.
    **kwargs: :ref:`userguide_styling_line_properties`

Returns:
    GlyphRenderer

Examples:

    .. bokeh-plot::
        :source-position: above

        from bokeh.plotting import figure, output_file, show

        plot = figure(width=300, height=300)
        plot.x(x=[1, 2, 3], y=[1, 2, 3], size=[10, 20, 25], color="#fa9fb5")

        show(plot)

"""
)



# The plot server must be running
# Go to http://localhost:5006/bokeh to view this plot

import numpy as np

from bokeh.models import BoxSelectTool, LassoSelectTool, Paragraph
from bokeh.plotting import (
    curdoc, cursession, figure, output_server, show, hplot, vplot
)

# create three normal population samples with different parameters
N1 = 2000
N2 = 5000
N3 = 1000

x1 = np.random.normal(loc=5.0, size=N1) * 100
y1 = np.random.normal(loc=10.0, size=N1) * 10

x2 = np.random.normal(loc=5.0, size=N2) * 50
y2 = np.random.normal(loc=5.0, size=N2) * 10

x3 = np.random.normal(loc=55.0, size=N3) * 10
y3 = np.random.normal(loc=4.0, size=N3) * 10

x = np.concatenate((x1, x2, x3))
y = np.concatenate((y1, y2, y3))

all_inds = np.arange(len(x1) + len(x2) + len(x3))

output_server("selection_histogram")

TOOLS="pan,wheel_zoom,box_select,lasso_select"

# create the scatter plot
p = figure(tools=TOOLS, plot_width=600, plot_height=600, title=None, min_border=10, min_border_left=50)
p.scatter(x, y, size=3, color="#3A5785", alpha=0.6, name="scatter")

renderer = p.select(dict(name="scatter"))
scatter_ds = renderer[0].data_source

box_select_tool = p.select(dict(type=BoxSelectTool))
box_select_tool.select_every_mousemove = False
lasso_select_tool = p.select(dict(type=LassoSelectTool))
lasso_select_tool.select_every_mousemove = False

# create the horizontal histogram
hhist, hedges = np.histogram(x, bins=20)
hzeros = np.zeros(len(hedges)-1)
hmax = max(hhist)*1.1

ph = figure(toolbar_location=None, plot_width=p.plot_width, plot_height=200, x_range=p.x_range,
            y_range=(-hmax, hmax), title=None, min_border=10, min_border_left=50)
ph.quad(bottom=0, left=hedges[:-1], right=hedges[1:], top=hhist, color="white", line_color="#3A5785")
ph.quad(bottom=0, left=hedges[:-1], right=hedges[1:], top=hzeros, color="#3A5785", alpha=0.5, line_color=None, name="hhist")
ph.quad(bottom=0, left=hedges[:-1], right=hedges[1:], top=hzeros, color="#3A5785", alpha=0.1, line_color=None, name="hhist2")
ph.xgrid.grid_line_color = None

ph_source = ph.select(dict(name="hhist"))[0].data_source
ph_source2 = ph.select(dict(name="hhist2"))[0].data_source

# create the vertical histogram
vhist, vedges = np.histogram(y, bins=20)
vzeros = np.zeros(len(vedges)-1)
vmax = max(vhist)*1.1

# need to adjust for toolbar height, unfortunately
th = 42

pv = figure(toolbar_location=None, plot_width=200, plot_height=p.plot_height+th-10, x_range=(-vmax, vmax),
            y_range=p.y_range, title=None, min_border=10, min_border_top=th)
pv.quad(left=0, bottom=vedges[:-1], top=vedges[1:], right=vhist, color="white", line_color="#3A5785")
pv.quad(left=0, bottom=vedges[:-1], top=vedges[1:], right=vzeros, color="#3A5785", alpha=0.5, line_color=None, name="vhist")
pv.quad(left=0, bottom=vedges[:-1], top=vedges[1:], right=vzeros, color="#3A5785", alpha=0.1, line_color=None, name="vhist2")
pv.ygrid.grid_line_color = None

pv_source = pv.select(dict(name="vhist"))[0].data_source
pv_source2 = pv.select(dict(name="vhist2"))[0].data_source

# set up callbacks
def on_selection_change(obj, attr, old, new):
    inds = np.array(new['1d']['indices'])
    if len(inds) == 0 or len(inds) == len(x):
        hhist = hzeros
        vhist = vzeros
        hhist2 = hzeros
        vhist2 = vzeros
    else:
        hhist, _ = np.histogram(x[inds], bins=hedges)
        vhist, _ = np.histogram(y[inds], bins=vedges)
        negative_inds = np.ones_like(x, dtype=np.bool)
        negative_inds[inds] = False
        hhist2, _ = np.histogram(x[negative_inds], bins=hedges)
        vhist2, _ = np.histogram(y[negative_inds], bins=vedges)

    ph_source.data["top"] = hhist
    pv_source.data["right"] = vhist
    ph_source2.data["top"] = -hhist2
    pv_source2.data["right"] = -vhist2

    cursession().store_objects(ph_source, pv_source, ph_source2, pv_source2)

scatter_ds.on_change('selected', on_selection_change)

layout = vplot(hplot(p, pv), hplot(ph, Paragraph()))
show(layout)

cursession().poll_document(curdoc(), 0.05)



from bokeh.plotting import figure, hplot, output_file, show
from bokeh.models import  ColumnDataSource, HoverTool, CustomJS, Slider
from bokeh.io import vform
import colorsys


# for plot 2: create colour spectrum of resolution N and brightness I, return as list of decimal RGB value tuples
def generate_color_range(N, I):
    HSV_tuples = [ (x*1.0/N, 0.5, I) for x in range(N) ]
    RGB_tuples = map(lambda x: colorsys.hsv_to_rgb(*x), HSV_tuples)
    for_conversion = []
    for RGB_tuple in RGB_tuples:
        for_conversion.append((int(RGB_tuple[0]*255), int(RGB_tuple[1]*255), int(RGB_tuple[2]*255)))
    hex_colors = [ rgb_to_hex(RGB_tuple) for RGB_tuple in for_conversion ]
    return hex_colors, for_conversion

# convert RGB tuple to hexadecimal code
def rgb_to_hex(rgb):
    return '#%02x%02x%02x' % rgb

# convert hexadecimal to RGB tuple
def hex_to_dec(hex):
    red = ''.join(hex.strip('#')[0:2])
    green = ''.join(hex.strip('#')[2:4])
    blue = ''.join(hex.strip('#')[4:6])
    return (int(red, 16), int(green, 16), int(blue,16))

# plot 1: create a color block with RGB values adjusted with sliders
# initialise a white block for the first plot
x = [0]
y = [0]
red   = 255
green = 255
blue  = 255
hex_color = rgb_to_hex((red,green,blue))

# initialise the text color as black. This will be switched to white if the block color gets dark enough
text_color = '#000000'

# create a data source to enable refreshing of fill & text color
source = ColumnDataSource(data=dict(x = x, y = y, color = [hex_color], text_color = [text_color]))

tools1 = 'reset, save'

# create first plot, as a rect() glyph and centered text label, with fill and text color taken from source
p1 = figure(x_range=(-8, 8), y_range=(-4, 4), plot_width=600, plot_height=300, title=None, tools=tools1)
color_block = p1.rect(x='x', y='y', width=18, height=10, fill_color='color', line_color = 'black', source=source)
hex_code_text = p1.text('x', 'y', text='color', text_color='text_color', alpha=0.6667, text_font_size='36pt', text_baseline='middle', text_align='center', source=source)

# the callback function to update the color of the block and associated label text
# NOTE: the JS functions for converting RGB to hex are taken from the excellent answer
# by Tim Down at http://stackoverflow.com/questions/5623838/rgb-to-hex-and-hex-to-rgb
callback = CustomJS(args=dict(source=source), code="""
    function componentToHex(c) {
        var hex = c.toString(16);
        return hex.length == 1 ? "0" + hex : hex;
    }
    function rgbToHex(r, g, b) {
        return "#" + componentToHex(r) + componentToHex(g) + componentToHex(b);
    }
    var data = source.get('data');
    var RS = red_slider;
    var GS = green_slider;
    var BS = blue_slider;
    color = data['color'];
    var red_from_slider = RS.get('value');
    var green_from_slider = GS.get('value');
    var blue_from_slider = BS.get('value');
    text_color = data['text_color'];
    color[0] = rgbToHex(red_from_slider, green_from_slider, blue_from_slider);
    if ((red_from_slider > 127) || (green_from_slider > 127) || (blue_from_slider > 127)) {
        text_color[0] = '#000000';
    }
    else {
        text_color[0] = '#ffffff';
    }
    source.trigger('change');
""")

# create slider tool objects to control the RGB levels for first plot. Set callback function to allow refresh
red_slider = Slider(start=0, end=255, value=255, step=1, title="R", callback=callback)
green_slider = Slider(start=0, end=255, value=255, step=1, title="G", callback=callback)
blue_slider = Slider(start=0, end=255, value=255, step=1, title="B", callback=callback)
callback.args['red_slider'] = red_slider
callback.args['green_slider'] = green_slider
callback.args['blue_slider'] = blue_slider

# plot 2: create a color spectrum with a hover-over tool to inspect hex codes
brightness = 0.8 # at the moment this is hard-coded. Change if you want brighter/darker colors
crx = list(range(1,1001)) # the resolution is 1000 colors
cry = [ 5 for i in range(len(crx)) ]
crcolor, crRGBs = generate_color_range(1000,brightness) # produce spectrum

# make data source object to allow information to be displayed by hover tool
crsource = ColumnDataSource(data=dict(x = crx, y = cry, crcolor = crcolor, RGBs = crRGBs))

tools2 = 'reset, save, hover'

# create second plot
p2 = figure(x_range=(0,1000), y_range=(0,10), plot_width=600, plot_height=150, tools=tools2, title = 'hover over color')
color_range1 = p2.rect(x='x', y='y', width=1, height=10, color='crcolor', source=crsource)
# set up hover tool to show color hex code and sample swatch
hover = p2.select(dict(type=HoverTool))
hover.tooltips = [
                   ('color', '$color[hex, rgb, swatch]:crcolor'),
                   ('RGB levels', '@RGBs')
                    ]

# get rid of axis details for cleaner look
p1.ygrid.grid_line_color = None
p1.xgrid.grid_line_color = None
p1.axis.axis_line_color  = None
p1.axis.major_label_text_color = None
p1.axis.major_tick_line_color = None
p1.axis.minor_tick_line_color = None

p2.ygrid.grid_line_color = None
p2.xgrid.grid_line_color = None
p2.axis.axis_line_color  = None
p2.axis.major_label_text_color = None
p2.axis.major_tick_line_color = None
p2.axis.minor_tick_line_color = None

layout = hplot(
    vform(red_slider, green_slider, blue_slider),
    vform(p1, p2)
)

output_file("color_sliders.html")
show(layout)



''' Define standard warning codes and messages for Bokeh validation checks.

1000 : *MISSING_RENDERERS*
    A |Plot| object has no renderers configured (will result in a blank plot).

1001 : *NO_GLYPH_RENDERERS*
    A |Plot| object has no glyph renderers (will result in an empty plot frame).

1002 : *EMPTY_LAYOUT*
    A layout model has no children (will result in a blank layout).

1003 : *MALFORMED_CATEGORY_LABEL*
    Category labels cannot contain colons (will result in a blank layout).

9999 : *EXT*
    Indicates that a custom warning check has failed.

'''

codes = {
    1000: ("MISSING_RENDERERS",         "Plot has no renderers"),
    1001: ("NO_GLYPH_RENDERERS",        "Plot has no glyph renderers"),
    1002: ("EMPTY_LAYOUT",              "Layout has no children"),
    1003: ("MALFORMED_CATEGORY_LABEL",  "Category labels cannot contain colons"),
    9999: ("EXT",                       "Custom extension reports warning"),
}

for code in codes:
    exec("%s = %d" % (codes[code][0], code))




# The plot server must be running
# Go to http://localhost:5006/bokeh to view this plot

import numpy as np
from scipy.integrate import odeint

from bokeh.plotting import figure, show, output_server

sigma = 10
rho = 28
beta = 8.0/3
theta = 3 * np.pi / 4

def lorenz(xyz, t):
    x, y, z = xyz
    x_dot = sigma * (y - x)
    y_dot = x * rho - x * z - y
    z_dot = x * y - beta* z
    return [x_dot, y_dot, z_dot]

initial = (-10, -7, 35)
t = np.arange(0, 100, 0.006)

solution = odeint(lorenz, initial, t)

x = solution[:, 0]
y = solution[:, 1]
z = solution[:, 2]
xprime = np.cos(theta) * x - np.sin(theta) * y

colors = ["#C6DBEF", "#9ECAE1", "#6BAED6", "#4292C6", "#2171B5", "#08519C", "#08306B",]

output_server("lorenz")

p = figure(title="lorenz example")

p.multi_line(np.array_split(xprime, 7), np.array_split(z, 7),
           line_color=colors, line_alpha=0.8, line_width=1.5)

show(p)  # open a browser



#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2015, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
from __future__ import absolute_import

from os import walk
from os.path import join

from bokeh.settings import settings
import flask

class BokehBlueprint(flask.Blueprint):

    def __init__(self, *args, **kwargs):
        super(BokehBlueprint, self).__init__(*args, **kwargs)
        self.debugjs = None

    def setup(self, backend, backbone_storage, servermodel_storage,
              authentication):
        self.backend = backend
        self.backbone_storage = backbone_storage
        self.servermodel_storage = servermodel_storage
        self.authentication = authentication
        self.bokehjsdir = settings.bokehjsdir()
        self.bokehjssrcdir = settings.bokehjssrcdir()

    def current_user(self):
        return self.authentication.current_user()

    def js_files(self):
        bokehjsdir = self.bokehjsdir
        js_files = []
        for root, dirnames, files in walk(bokehjsdir):
            for fname in files:
                if fname.endswith(".js") and 'vendor' not in root:
                    js_files.append(join(root, fname))
        return js_files

bokeh_app = BokehBlueprint(
    'bokeh.server',
    'bokeh.server',
    static_folder='static',
    static_url_path='/bokeh/static',
    template_folder='_templates'
)

app = flask.Flask("bokeh.server")



from __future__ import print_function

from math import pi
import pandas as pd

from bokeh.models import Plot, ColumnDataSource, FactorRange, CategoricalAxis, TapTool, HoverTool, OpenURL
from bokeh.models.glyphs import Rect
from bokeh.document import Document
from bokeh.embed import file_html
from bokeh.resources import INLINE
from bokeh.browserlib import view

css3_colors = pd.DataFrame([
    ("Pink",                  "#FFC0CB", "Pink"),
    ("LightPink",             "#FFB6C1", "Pink"),
    ("HotPink",               "#FF69B4", "Pink"),
    ("DeepPink",              "#FF1493", "Pink"),
    ("PaleVioletRed",         "#DB7093", "Pink"),
    ("MediumVioletRed",       "#C71585", "Pink"),
    ("LightSalmon",           "#FFA07A", "Red"),
    ("Salmon",                "#FA8072", "Red"),
    ("DarkSalmon",            "#E9967A", "Red"),
    ("LightCoral",            "#F08080", "Red"),
    ("IndianRed",             "#CD5C5C", "Red"),
    ("Crimson",               "#DC143C", "Red"),
    ("FireBrick",             "#B22222", "Red"),
    ("DarkRed",               "#8B0000", "Red"),
    ("Red",                   "#FF0000", "Red"),
    ("OrangeRed",             "#FF4500", "Orange"),
    ("Tomato",                "#FF6347", "Orange"),
    ("Coral",                 "#FF7F50", "Orange"),
    ("DarkOrange",            "#FF8C00", "Orange"),
    ("Orange",                "#FFA500", "Orange"),
    ("Yellow",                "#FFFF00", "Yellow"),
    ("LightYellow",           "#FFFFE0", "Yellow"),
    ("LemonChiffon",          "#FFFACD", "Yellow"),
    ("LightGoldenrodYellow",  "#FAFAD2", "Yellow"),
    ("PapayaWhip",            "#FFEFD5", "Yellow"),
    ("Moccasin",              "#FFE4B5", "Yellow"),
    ("PeachPuff",             "#FFDAB9", "Yellow"),
    ("PaleGoldenrod",         "#EEE8AA", "Yellow"),
    ("Khaki",                 "#F0E68C", "Yellow"),
    ("DarkKhaki",             "#BDB76B", "Yellow"),
    ("Gold",                  "#FFD700", "Yellow"),
    ("Cornsilk",              "#FFF8DC", "Brown"),
    ("BlanchedAlmond",        "#FFEBCD", "Brown"),
    ("Bisque",                "#FFE4C4", "Brown"),
    ("NavajoWhite",           "#FFDEAD", "Brown"),
    ("Wheat",                 "#F5DEB3", "Brown"),
    ("BurlyWood",             "#DEB887", "Brown"),
    ("Tan",                   "#D2B48C", "Brown"),
    ("RosyBrown",             "#BC8F8F", "Brown"),
    ("SandyBrown",            "#F4A460", "Brown"),
    ("Goldenrod",             "#DAA520", "Brown"),
    ("DarkGoldenrod",         "#B8860B", "Brown"),
    ("Peru",                  "#CD853F", "Brown"),
    ("Chocolate",             "#D2691E", "Brown"),
    ("SaddleBrown",           "#8B4513", "Brown"),
    ("Sienna",                "#A0522D", "Brown"),
    ("Brown",                 "#A52A2A", "Brown"),
    ("Maroon",                "#800000", "Brown"),
    ("DarkOliveGreen",        "#556B2F", "Green"),
    ("Olive",                 "#808000", "Green"),
    ("OliveDrab",             "#6B8E23", "Green"),
    ("YellowGreen",           "#9ACD32", "Green"),
    ("LimeGreen",             "#32CD32", "Green"),
    ("Lime",                  "#00FF00", "Green"),
    ("LawnGreen",             "#7CFC00", "Green"),
    ("Chartreuse",            "#7FFF00", "Green"),
    ("GreenYellow",           "#ADFF2F", "Green"),
    ("SpringGreen",           "#00FF7F", "Green"),
    ("MediumSpringGreen",     "#00FA9A", "Green"),
    ("LightGreen",            "#90EE90", "Green"),
    ("PaleGreen",             "#98FB98", "Green"),
    ("DarkSeaGreen",          "#8FBC8F", "Green"),
    ("MediumSeaGreen",        "#3CB371", "Green"),
    ("SeaGreen",              "#2E8B57", "Green"),
    ("ForestGreen",           "#228B22", "Green"),
    ("Green",                 "#008000", "Green"),
    ("DarkGreen",             "#006400", "Green"),
    ("MediumAquamarine",      "#66CDAA", "Cyan"),
    ("Aqua",                  "#00FFFF", "Cyan"),
    ("Cyan",                  "#00FFFF", "Cyan"),
    ("LightCyan",             "#E0FFFF", "Cyan"),
    ("PaleTurquoise",         "#AFEEEE", "Cyan"),
    ("Aquamarine",            "#7FFFD4", "Cyan"),
    ("Turquoise",             "#40E0D0", "Cyan"),
    ("MediumTurquoise",       "#48D1CC", "Cyan"),
    ("DarkTurquoise",         "#00CED1", "Cyan"),
    ("LightSeaGreen",         "#20B2AA", "Cyan"),
    ("CadetBlue",             "#5F9EA0", "Cyan"),
    ("DarkCyan",              "#008B8B", "Cyan"),
    ("Teal",                  "#008080", "Cyan"),
    ("LightSteelBlue",        "#B0C4DE", "Blue"),
    ("PowderBlue",            "#B0E0E6", "Blue"),
    ("LightBlue",             "#ADD8E6", "Blue"),
    ("SkyBlue",               "#87CEEB", "Blue"),
    ("LightSkyBlue",          "#87CEFA", "Blue"),
    ("DeepSkyBlue",           "#00BFFF", "Blue"),
    ("DodgerBlue",            "#1E90FF", "Blue"),
    ("CornflowerBlue",        "#6495ED", "Blue"),
    ("SteelBlue",             "#4682B4", "Blue"),
    ("RoyalBlue",             "#4169E1", "Blue"),
    ("Blue",                  "#0000FF", "Blue"),
    ("MediumBlue",            "#0000CD", "Blue"),
    ("DarkBlue",              "#00008B", "Blue"),
    ("Navy",                  "#000080", "Blue"),
    ("MidnightBlue",          "#191970", "Blue"),
    ("Lavender",              "#E6E6FA", "Purple"),
    ("Thistle",               "#D8BFD8", "Purple"),
    ("Plum",                  "#DDA0DD", "Purple"),
    ("Violet",                "#EE82EE", "Purple"),
    ("Orchid",                "#DA70D6", "Purple"),
    ("Fuchsia",               "#FF00FF", "Purple"),
    ("Magenta",               "#FF00FF", "Purple"),
    ("MediumOrchid",          "#BA55D3", "Purple"),
    ("MediumPurple",          "#9370DB", "Purple"),
    ("BlueViolet",            "#8A2BE2", "Purple"),
    ("DarkViolet",            "#9400D3", "Purple"),
    ("DarkOrchid",            "#9932CC", "Purple"),
    ("DarkMagenta",           "#8B008B", "Purple"),
    ("Purple",                "#800080", "Purple"),
    ("Indigo",                "#4B0082", "Purple"),
    ("DarkSlateBlue",         "#483D8B", "Purple"),
    ("SlateBlue",             "#6A5ACD", "Purple"),
    ("MediumSlateBlue",       "#7B68EE", "Purple"),
    ("White",                 "#FFFFFF", "White"),
    ("Snow",                  "#FFFAFA", "White"),
    ("Honeydew",              "#F0FFF0", "White"),
    ("MintCream",             "#F5FFFA", "White"),
    ("Azure",                 "#F0FFFF", "White"),
    ("AliceBlue",             "#F0F8FF", "White"),
    ("GhostWhite",            "#F8F8FF", "White"),
    ("WhiteSmoke",            "#F5F5F5", "White"),
    ("Seashell",              "#FFF5EE", "White"),
    ("Beige",                 "#F5F5DC", "White"),
    ("OldLace",               "#FDF5E6", "White"),
    ("FloralWhite",           "#FFFAF0", "White"),
    ("Ivory",                 "#FFFFF0", "White"),
    ("AntiqueWhite",          "#FAEBD7", "White"),
    ("Linen",                 "#FAF0E6", "White"),
    ("LavenderBlush",         "#FFF0F5", "White"),
    ("MistyRose",             "#FFE4E1", "White"),
    ("Gainsboro",             "#DCDCDC", "Gray/Black"),
    ("LightGray",             "#D3D3D3", "Gray/Black"),
    ("Silver",                "#C0C0C0", "Gray/Black"),
    ("DarkGray",              "#A9A9A9", "Gray/Black"),
    ("Gray",                  "#808080", "Gray/Black"),
    ("DimGray",               "#696969", "Gray/Black"),
    ("LightSlateGray",        "#778899", "Gray/Black"),
    ("SlateGray",             "#708090", "Gray/Black"),
    ("DarkSlateGray",         "#2F4F4F", "Gray/Black"),
    ("Black",                 "#000000", "Gray/Black"),
], columns=["Name", "Color", "Group"])

source = ColumnDataSource(dict(
    names  = list(css3_colors.Name),
    groups = list(css3_colors.Group),
    colors = list(css3_colors.Color),
))

xdr = FactorRange(factors=list(css3_colors.Group.unique()))
ydr = FactorRange(factors=list(reversed(css3_colors.Name)))

plot = Plot(title="CSS3 Color Names", x_range=xdr, y_range=ydr, plot_width=600, plot_height=2000)

rect = Rect(x="groups", y="names", width=1, height=1, fill_color="colors", line_color=None)
rect_renderer = plot.add_glyph(source, rect)

xaxis_above = CategoricalAxis(major_label_orientation=pi/4)
plot.add_layout(xaxis_above, 'above')

xaxis_below = CategoricalAxis(major_label_orientation=pi/4)
plot.add_layout(xaxis_below, 'below')

plot.add_layout(CategoricalAxis(), 'left')

url = "http://www.colors.commutercreative.com/@names/"
tooltips = """Click the color to go to:<br /><a href="{url}">{url}</a>""".format(url=url)

tap = TapTool(plot=plot, renderers=[rect_renderer], callback=OpenURL(url=url))
hover = HoverTool(plot=plot, renderers=[rect_renderer], tooltips=tooltips)
plot.tools.extend([tap, hover])

doc = Document()
doc.add(plot)

if __name__ == "__main__":
    filename = "colors.html"
    with open(filename, "w") as f:
        f.write(file_html(doc, INLINE, "CSS3 Color Names"))
    print("Wrote %s" % filename)
    view(filename)



#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2015, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
from __future__ import absolute_import

from bokeh.exceptions import AuthenticationException
from bokeh.models.plots import Plot
from bokeh.session import TestSession as Session
from werkzeug.exceptions import Unauthorized

from bokeh.server.app import app, bokeh_app
from bokeh.server.models.user import User, new_user
from bokeh.server.models.docs import Doc
from bokeh.server.serverbb import BokehServerTransaction
from bokeh.server.views.bbauth import handle_auth_error
from bokeh.server.views.main import _makedoc
from bokeh.util.testing import FlaskClientTestCase


class AuthTestCase(FlaskClientTestCase):
    options = {'multi_user' : True}
    def test_handle_auth_error_decorator(self):
        @handle_auth_error
        def helper1():
            return 'foo'
        @handle_auth_error
        def helper2():
            raise AuthenticationException('bad!')
        assert helper1() == 'foo'
        self.assertRaises(Unauthorized, helper2)

    def test_write_auth_checks_rw_users_field(self):
        user = User('test1', 'sdfsdf', 'sdfsdf')
        user2 = User('test2', 'sdfsdf', 'sdfsdf')
        doc = Doc('docid', 'title', ['test1'], [], None,
                  None, None, False)
        with app.test_request_context("/"):
            BokehServerTransaction(user, doc, 'rw')
        with app.test_request_context("/"):
            self.assertRaises(AuthenticationException, BokehServerTransaction,
                              user2, doc, 'rw')

    def test_write_auth_ignores_r_users_field(self):
        user2 = User('test2', 'sdfsdf', 'sdfsdf')
        doc = Doc('docid', 'title', ['test1'], ['test'], None,
                  None, None, False)
        with app.test_request_context("/"):
            self.assertRaises(AuthenticationException, BokehServerTransaction,
                              user2, doc, 'rw')

    def test_read_auth_checks_both_fields(self):
        user2 = User('test2', 'sdfsdf', 'sdfsdf')
        user = User('test1', 'sdfsdf', 'sdfsdf')
        doc = Doc('docid', 'title', ['test1'], ['test2'], None,
                  None, None, False)
        with app.test_request_context("/"):
            BokehServerTransaction(user, doc, 'r')
        with app.test_request_context("/"):
            BokehServerTransaction(user2, doc, 'r')

    def test_permissions_with_temporary_docid(self):
        user = User('test1', 'sdfsdf', 'sdfsdf')
        doc = Doc('docid', 'title', [], ['test2'], None,
                  None, None, False)
        # with temporary docid, a user must be able to read in order to get write access
        # this call should fail
        with app.test_request_context("/"):
            self.assertRaises(AuthenticationException, BokehServerTransaction,
                              user, doc, 'rw', temporary_docid="foobar")

class TransactionManagerTestCase(FlaskClientTestCase):
    options = {'multi_user' : True}
    def setUp(self):
        super(TransactionManagerTestCase, self).setUp()
        self.user = new_user(bokeh_app.servermodel_storage, "test1",
                        "password")
        self.server_docobj = _makedoc(
            bokeh_app.servermodel_storage, self.user, 'testdoc1'
        )
        #create 2 views
        original_doc = bokeh_app.backbone_storage.get_document(self.server_docobj.docid)
        plot1 = Plot(title='plot1')
        original_doc.add(plot1)
        bokeh_app.backbone_storage.store_document(original_doc)

    def transaction(self, temporary_docid, mode='rw'):
        context = BokehServerTransaction(
            self.user,
            self.server_docobj,
            mode,
            temporary_docid=temporary_docid)
        return context

    def test_transaction_with_anonymous_user(self):
        self.server_docobj.published = True
        with app.test_request_context("/"):
            self.assertRaises(
                AuthenticationException,
                BokehServerTransaction,
                None, self.server_docobj, 'rw'
            )
        with app.test_request_context("/"):
            doc = BokehServerTransaction(None, self.server_docobj, 'rw',
                                         temporary_docid='temp')
            assert doc

    def test_base_object_exists_in_cow_context(self):
        with app.test_request_context("/"):
            t = self.transaction('test1')
            t.load()
            assert t.clientdoc.context.children[0].title == 'plot1'

    def test_writing_in_cow_context_persists(self):
        with app.test_request_context("/"):
            t = self.transaction('test1')
            t.load()
            t.clientdoc.add(Plot(title='plot2'))
            t.save()

            t = self.transaction('test1')
            t.load()
            assert len(t.clientdoc.context.children) == 2
            assert t.clientdoc.context.children[1].title == 'plot2'

    def test_writing_in_cow_context_does_not_modify_original(self):
        with app.test_request_context("/"):
            t = self.transaction('test1')
            t.load()
            t.clientdoc.add(Plot(title='plot2'))
            t.save()

            t = self.transaction('test1')
            t.load()
            assert len(t.clientdoc.context.children) == 2

            t = self.transaction(None)
            t.load()
            assert len(t.clientdoc.context.children) == 1

    def test_cannot_save_to_read_only_context(self):
        with app.test_request_context("/"):
            t = self.transaction(None, mode='r')
            t.load()
            t.clientdoc.add(Plot(title='plot2'))
            self.assertRaises(AuthenticationException, t.save)

    def test_cannot_gc_read_only_context(self):
        with app.test_request_context("/"):
            t = self.transaction(None, mode='r')
            self.assertRaises(AuthenticationException, t.load, gc=True)

class PublishTestCase(FlaskClientTestCase):
    options = {'multi_user' : True}
    def test_publish(self):
        sess = Session(client=app.test_client())
        sess.register('testuser', 'testpassword')
        sess.use_doc('test_cow')
        sess.publish()
        doc = Doc.load(bokeh_app.servermodel_storage, sess.docid)
        assert doc.title == 'test_cow'
        assert doc.published == True

    def test_publish_fails_for_invalid_auth(self):
        sess = Session(client=app.test_client())
        sess.register('testuser', 'testpassword')
        sess.use_doc('test_cow')
        sess2 = Session(client=app.test_client())
        sess2.register('testuser2', 'testpassword')
        sess2.docid = sess.docid
        self.assertRaises(Exception, sess2.publish)



"""This is the Bokeh charts interface. It gives you a high level API to build
complex plot is a simple way.

This is the ChartObject class, a minimal prototype class to build more chart
types on top of it. It provides the mechanisms to support the shared chained
methods.
"""
# -----------------------------------------------------------------------------
# Copyright (c) 2012 - 2014, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
# -----------------------------------------------------------------------------

from __future__ import absolute_import

from six import iteritems
from six.moves import zip
from copy import copy
from operator import itemgetter
from itertools import islice, product, chain
import numpy as np
import pandas as pd

from ..properties import bokeh_integer_types, Datetime, List, HasProps
from ..models.sources import ColumnDataSource

from ._properties import ColumnLabel
from .utils import collect_attribute_columns, special_columns

DEFAULT_COLUMN_NAMES = 'abcdefghijklmnopqrstuvwxyz'
COMPUTED_COLUMN_NAMES = ['_charts_ones']
ARRAY_TYPES = [tuple, list, np.ndarray, pd.Series]
TABLE_TYPES = [dict, pd.DataFrame]


def take(n, iterable):
    """Return first n items of the iterable as a list."""
    return islice(iterable, n)


def gen_column_names(n):
    """Produces list of unique column names of length n."""
    col_names = list(DEFAULT_COLUMN_NAMES)

    # a-z
    if n < len(col_names):
        return list(take(n, col_names))
    # a-z and aa-zz (500+ columns)
    else:
        n_left = n - len(col_names)
        labels = [''.join(item) for item in
                  take(n_left, product(DEFAULT_COLUMN_NAMES, DEFAULT_COLUMN_NAMES))]
        col_names.extend(labels)
        return col_names


class DataOperator(HasProps):
    columns = List(ColumnLabel(), default=None,
                   help="""List of columns to perform operation on.""")

    def apply(self, data):
        raise NotImplementedError('Each data operator must implement the apply method.')

    def __repr__(self):
        col_str = ', '.join(self.columns)
        return '%s(%s)' % (self.__class__.__name__, col_str)


class DataGroup(object):
    """Contains subset of data and metadata about it."""

    def __init__(self, label, data, attr_specs):
        self.label = label
        self.data = data.reset_index()
        self.attr_specs = attr_specs

    def get_values(self, selection):
        if selection in special_columns:
            return special_columns[selection](self.data)
        elif isinstance(selection, str):
            return self.data[selection]
        elif isinstance(selection, list) and len(selection) == 1:
            return self.data[selection[0]]
        elif isinstance(selection, list) and len(selection) > 1:
            return self.data[selection]
        else:
            return None

    @property
    def source(self):
        return ColumnDataSource(self.data)

    def __getitem__(self, key):
        return self.attr_specs[key]

    def __repr__(self):
        return '<DataGroup(%s) - attributes: %s>' % (str(self.label), self.attr_specs)

    def __len__(self):
        return len(self.data.index)


def groupby(df, **specs):
    """Convenience iterator around pandas groupby and attribute specs."""

    spec_cols = collect_attribute_columns(**specs)

    # if there was any input for chart attributes, which require grouping
    if spec_cols:
        # df = df.sort(columns=spec_cols)

        for name, data in df.groupby(spec_cols):

            attrs = {}
            for spec_name, spec in iteritems(specs):
                if spec.columns is not None:
                    # get index of the unique column values grouped on for this spec
                    name_idx = tuple([spec_cols.index(col) for col in spec.columns])

                    if isinstance(name, tuple):
                        # this handles the case of utilizing one or more and overlapping
                        # column names for different attrs
                        # name (label) is a tuple of the column values
                        # we extract only the data associated with the columns that this attr spec was configured with
                        label = itemgetter(*name_idx)(name)
                    else:
                        label = name
                else:
                    label = None

                # get attribute value for this spec, given the unique column values associated with it
                attrs[spec_name] = spec[label]

            yield DataGroup(label=name, data=data, attr_specs=attrs)

    # collect up the defaults from the attribute specs
    else:
        attrs = {}
        for spec_name, spec in iteritems(specs):
            attrs[spec_name] = spec[None]

        yield DataGroup(label='all', data=df, attr_specs=attrs)


DEFAULT_DIMS = ['x', 'y']
DEFAULT_REQ_DIMS = [['x'], ['y'], ['x', 'y']]


class ChartDataSource(object):
    """Validates, normalizes, groups, and assigns Chart attributes to groups.

    Supported inputs are:
        Array-like - list, tuple, np.ndarray, pd.Series
        Table-like - records: list(dict), columns: dict(list), pd.DataFrame, blaze resource

    Converts inputs that could be treated as table-like data to pandas
    DataFrame, which is used for assigning attributes to data groups.
    """

    def __init__(self, df, dims=None, required_dims=None, selections=None, **kwargs):

        if dims is None:
            dims = DEFAULT_DIMS

        if required_dims is None:
            required_dims = DEFAULT_REQ_DIMS

        self._data = df
        self._dims = dims
        self._required_dims = required_dims
        self._selections = self.get_selections(selections, **kwargs)
        self.apply_operations()
        self.meta = self.collect_metadata(df)
        self._validate_selections()

    def get_selections(self, selections, **kwargs):
        """Maps chart dimensions to selections and checks that required dim requirements are met."""
        select_map = {}

        # extract selections from kwargs using dimension list
        for dim in self._dims:
            dim_select = kwargs.pop(dim, None)
            if dim_select is not None:
                select_map[dim] = dim_select

        # handle case where dimension kwargs were not provided
        if len(select_map.keys()) == 0:
            if selections is None:
                # if no selections are provided, we assume they were provided in order
                select_map = {dim: sel for dim, sel in
                              zip(self._dims, self._data.columns)}
            elif isinstance(selections, dict):
                if len(selections.keys()) != 0:
                    # selections were specified in inputs
                    select_map = selections
            else:
                # selection input type isn't valid
                raise ValueError(
                    'selections input must be provided as: dict(dimension: column) or None')

        # make sure each dimension is represented in the selection map
        for dim in self._dims:
            if dim not in select_map:
                select_map[dim] = None

        # # make sure we have enough dimensions as required either way
        # unmatched = list(set(self._required_dims) - set(select_map.keys()))
        # if len(unmatched) > 0:
        #     raise ValueError('You must provide all required columns assigned to dimensions, no match for: %s'
        #                      % ', '.join(unmatched))
        # else:
        return select_map

    def apply_operations(self):
        # ToDo: Handle order of operation application, see GoG pg. 71
        for dim, select in iteritems(self._selections):
            if isinstance(select, DataOperator):
                self._data = select.apply(self)

    def __getitem__(self, dim):
        """Get the columns selected for the given dimension name.

        e.g. dim='x'
        """
        if dim in self._selections:
            return self._selections[dim]
        else:
            return None

    def stack_measures(self, measures, ids=None, var_name='variable', value_name='value'):
        for dim in self._dims:
            # find the dimension the measures are associated with

            selection = self._selections[dim]
            if isinstance(selection, DataOperator):
                dim_cols = selection.columns
            else:
                dim_cols = selection

            if measures == dim_cols:
                self._selections[dim] = value_name
                if ids is not None:
                    self._data = pd.melt(self._data, id_vars=ids, value_vars=measures,
                                         var_name=var_name, value_name=value_name)
                else:
                    ids = list(set(self._data.columns) - set(measures))
                    self._data = pd.melt(self._data, id_vars=ids, value_vars=measures,
                                         var_name=var_name, value_name=value_name)

    def groupby(self, **specs):
        """Iterable of chart attribute specifications, associated with columns.

        Iterates over DataGroup, which represent the lowest level of data that is assigned
        to the attributes for plotting.
        """
        if len(specs) == 0:
            raise ValueError(
                'You must provide one or more Attribute Specs to support iteration.')

        return groupby(self._data, **specs)

    @classmethod
    def from_data(cls, *args, **kwargs):
        """Automatically handle all valid inputs."""
        arrays = [arg for arg in args if cls.is_array(arg)]
        tables = [arg for arg in args if cls.is_table(arg) or cls.is_list_dicts(arg)]

        # only accept array-like or table-like input for simplicity
        if len(arrays) > 0 and len(tables) > 0:
            raise TypeError('Only input either array or table data.')

        # kwarg data
        if len(arrays) == 0 and len(tables) == 0:

            # handle list of lists
            list_dims = [k for k, v in iteritems(kwargs) if (cls.is_list_arrays(v) or
                                                             cls.is_array(v)) and
                         k is not 'dims' and k is not 'required_dims']
            if len(list_dims) > 0:
                arrays = [kwargs[dim] for dim in list_dims]
                if cls.is_list_arrays(arrays):
                    arrays = list(chain.from_iterable(arrays))
                col_names = gen_column_names(len(arrays))

                # reassign kwargs to new columns
                new_kwargs = kwargs.copy()
                for dim in list_dims:
                    dim_cols = []
                    dim_inputs = kwargs[dim]
                    if not cls.is_list_arrays(dim_inputs) and not all([cls.is_array(
                            dim_input) for dim_input in dim_inputs]):
                        dim_inputs = [dim_inputs]

                    # if we passed one to many literal array/list, match to cols
                    for dim_input in dim_inputs:
                        for array, col_name in zip(arrays, col_names):
                            if pd.Series.all(pd.Series(array) == pd.Series(dim_input)):
                                # add col to all cols and
                                dim_cols.append(col_name)

                    # if only single column selected, pull it out of list
                    if len(dim_cols) == 1:
                        dim_cols = dim_cols[0]

                    new_kwargs[dim] = dim_cols

                # setup kwargs to process as if we received arrays as args
                kwargs = new_kwargs
                kwargs['columns'] = col_names
            else:
                raise ValueError(
                    'No data found for inputs %s' % ', '.join(kwargs['dims']))

        # handle array-like
        if len(arrays) > 0:
            return cls.from_arrays(arrays, **kwargs)

        # handle table-like
        elif len(tables) > 0:

            # single table input only
            if len(tables) != 1:
                raise TypeError('Input a single table data type.')
            else:
                table = tables[0]

            # dict of arrays
            if isinstance(table, dict):
                if all([cls.is_array(col) for col in table.values()]):
                    return cls(df=pd.DataFrame.from_dict(data=table), **kwargs)
                else:
                    raise TypeError('Input of table-like dict must be column-oriented.')

            # list of dicts
            elif cls.is_list_dicts(table):
                return cls(df=pd.DataFrame.from_records(data=table), **kwargs)

            # blaze data source
            # elif string or datasource
            # Todo: implement handling of blaze data sources if available

            # pandas dataframe
            elif isinstance(table, pd.DataFrame):
                return cls(df=table, **kwargs)

            # unrecognized input type
            else:
                raise TypeError(
                    'Unable to recognize inputs for conversion to dataframe for %s'
                    % type(table))

    @staticmethod
    def is_list_arrays(data):
        valid = False

        # ToDo: handle groups of arrays types, list of lists of arrays
        # avoid case where we have a list with one list of values in it
        if (isinstance(data, list) and len(data) == 1 and isinstance(data[0], list) and
                not isinstance(data[0][0], list) and not ChartDataSource.is_array(data[0][0])):
            return valid

        # really want to check for nested lists, where each list might have lists
        if isinstance(data, list):
            if all([ChartDataSource.is_array(col) for col in data]):
                valid = True

        return valid

    @property
    def df(self):
        return self._data

    @staticmethod
    def _collect_dimensions(**kwargs):
        dims = kwargs.pop(kwargs, None)
        if not dims:
            return 'x', 'y'
        else:
            return dims

    @classmethod
    def from_arrays(cls, arrays, column_names=None, **kwargs):

        # handle list of arrays
        if any(cls.is_list_arrays(array) for array in arrays):
            list_of_arrays = copy(arrays)
            arrays = list(chain.from_iterable(arrays))
            column_names = column_names or gen_column_names(len(arrays))
            cols = copy(column_names)
            dims = kwargs.get('dims', None) or DEFAULT_DIMS

            # derive column selections
            for dim, list_of_array in zip(dims, list_of_arrays):
                sel = [cols.pop(0) for _ in list_of_array]
                kwargs[dim] = sel
        else:
            column_names = column_names or gen_column_names(len(arrays))

        # try to replace auto names with Series names
        for i, array in enumerate(arrays):
            if isinstance(array, pd.Series):
                name = array.name
                if name not in column_names:
                    column_names[i] = name

        table = {column_name: array for column_name, array in zip(column_names, arrays)}
        return cls(df=pd.DataFrame.from_dict(data=table), **kwargs)

    @classmethod
    def from_dict(cls, data, **kwargs):
        return cls(df=pd.DataFrame.from_dict(data), **kwargs)

    @staticmethod
    def is_table(data):
        return ChartDataSource._is_valid(data,
                                         TABLE_TYPES) or ChartDataSource.is_list_dicts(
            data)

    @staticmethod
    def is_list_dicts(data):
        return isinstance(data, list) and all([isinstance(row, dict) for row in data])

    @staticmethod
    def is_array(data):
        if ChartDataSource.is_list_dicts(data):
            # list of dicts is table type
            return False
        else:
            return ChartDataSource._is_valid(data, ARRAY_TYPES)

    @staticmethod
    def _is_valid(data, types):
        return any([isinstance(data, valid_type) for valid_type in types])

    def _validate_selections(self):
        """Raises selection error if selections are not valid compared to requirements."""

        required_dims = self._required_dims
        selections = self._selections
        dims = [dim for dim, sel in iteritems(selections) if sel is not None]

        # look for a match for selections to dimensional requirements
        if len(required_dims) > 0:
            for req in required_dims:
                # ToDo: handle column type specifications

                if len(dims) < len(req):
                    # not an exact match
                    continue

                if all([dim in req for dim in dims]):
                    # found a match to the requirements
                    return

            # If we reach this point, nothing was validated, let's construct useful error messages
            error_str = 'Did not receive a valid combination of selections.\n\nValid configurations are: %s' + \
                        '\nReceived inputs are: %s' + \
                        '\n\nAvailable columns are: %s'
            req_str = [' and '.join(['%s = <Any Column>' % dim for dim in required_dim])
                       for required_dim in required_dims]
            selection_str = ['%s = %s' % (str(dim), str(sel)) for dim, sel in
                             iteritems(selections) if sel is not None]

            raise ValueError(error_str % (
            ' or '.join(req_str), ', '.join(selection_str), ', '.join(self.columns)))
        else:
            # if we have no dimensional requirements, they all pass
            return

    @staticmethod
    def is_number(value):
        numbers = (float,) + bokeh_integer_types
        return isinstance(value, numbers)

    @staticmethod
    def is_datetime(value):
        try:
            dt = Datetime(value)
            dt  # shut up pyflakes
            return True

        except ValueError:
            return False

    @staticmethod
    def collect_metadata(data):
        # ToDo: implement column metadata collection
        return {}

    @property
    def columns(self):
        return self._data.columns

    @property
    def index(self):
        return self._data.index

    @property
    def values(self):
        return self._data.values

    @staticmethod
    def is_computed(column):
        if column in COMPUTED_COLUMN_NAMES:
            return True
        else:
            return False



# The plot server must be running
# Go to http://localhost:5006/bokeh to view this plot

from __future__ import division

import numpy as np

from bokeh.plotting import figure, show, output_server

N = 20
img = np.empty((N,N), dtype=np.uint32)
view = img.view(dtype=np.uint8).reshape((N, N, 4))
for i in range(N):
    for j in range(N):
        view[i, j, 0] = int(i/N*255)
        view[i, j, 1] = 158
        view[i, j, 2] = int(j/N*255)
        view[i, j, 3] = 255

output_server("image_rgba")

p = figure(x_range=[0,10], y_range=[0,10])
p.image_rgba(image=[img], x=[0], y=[0], dw=[10], dh=[10])

show(p)  # open a browser



#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2015, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
from __future__ import absolute_import

import logging

import imp
from os.path import dirname
import sys

from six.moves.queue import Queue
from tornado.web import Application, FallbackHandler
from tornado.wsgi import WSGIContainer

from .app import bokeh_app, app
from .models import convenience as mconv
from .models import docs
from .settings import settings as server_settings
from .websocket import WebSocketHandler, WebSocketManager
from .zmq import Forwarder, Publisher, Subscriber

from .server_backends import MultiUserAuthentication, SingleUserAuthentication

REDIS_PORT = 6379

def configure_flask(config_argparse=None, config_file=None, config_dict=None):
    if config_argparse:
        server_settings.from_args(config_argparse)
    if config_dict:
        server_settings.from_dict(config_dict)
    if config_file:
        server_settings.from_file(config_file)
    for handler in logging.getLogger().handlers:
        handler.addFilter(StaticFilter())
    # must import views before running apps
    from .views import deps
    # this just shuts up pyflakes
    deps
    backend = server_settings.model_backend
    if backend['type'] == 'redis':
        import redis
        from .storage.redis import RedisModelStorage, RedisBackboneStorage
        rhost = backend.get('redis_host', '127.0.0.1')
        rport = backend.get('redis_port', REDIS_PORT)
        bbstorage = RedisBackboneStorage(redis.Redis(host=rhost, port=rport, db=2))
        servermodel_storage = RedisModelStorage(redis.Redis(host=rhost, port=rport, db=3))

    elif backend['type'] == 'memory':
        from .storage.in_memory import InMemoryModelStorage, InMemoryBackboneStorage
        bbstorage = InMemoryBackboneStorage()
        servermodel_storage = InMemoryModelStorage()

    elif backend['type'] == 'shelve':
        from .storage.shelve import ShelveModelStorage, ShelveBackboneStorage
        bbstorage = ShelveBackboneStorage()
        servermodel_storage = ShelveModelStorage()

    if not server_settings.multi_user:
        authentication = SingleUserAuthentication()
    else:
        authentication = MultiUserAuthentication()
    bokeh_app.url_prefix = server_settings.url_prefix
    bokeh_app.publisher = Publisher(server_settings.ctx, server_settings.pub_zmqaddr, Queue())

    for script in server_settings.scripts:
        script_dir = dirname(script)
        if script_dir not in sys.path:
            print ("adding %s to python path" % script_dir)
            sys.path.append(script_dir)
        print ("importing %s" % script)
        imp.load_source("_bokeh_app", script)

    #todo - push some of this into bokeh_app.setup?
    bokeh_app.setup(
        backend,
        bbstorage,
        servermodel_storage,
        authentication,
    )
registered = False
def register_blueprint():
    global registered
    if registered:
        logging.warn(
            "register_blueprint has already been called, why is it being called again"
        )
        return
    app.register_blueprint(bokeh_app, url_prefix=server_settings.url_prefix)
    registered = True

class SimpleBokehTornadoApp(Application):
    def __init__(self, flask_app, **settings):
        self.flask_app = flask_app
        tornado_flask = WSGIContainer(flask_app)
        url_prefix = server_settings.url_prefix
        handlers = [
            (url_prefix + "/bokeh/sub", WebSocketHandler),
            (r".*", FallbackHandler, dict(fallback=tornado_flask))
        ]
        super(SimpleBokehTornadoApp, self).__init__(handlers, **settings)
        self.wsmanager = WebSocketManager()
        def auth(auth, docid):
            #HACKY
            if docid.startswith("temporary-"):
                return True
            doc = docs.Doc.load(bokeh_app.servermodel_storage, docid)
            status = mconv.can_read_doc_api(doc, auth)
            return status
        self.wsmanager.register_auth('bokehplot', auth)

        self.subscriber = Subscriber(server_settings.ctx,
                                     [server_settings.sub_zmqaddr],
                                     self.wsmanager)
        if server_settings.run_forwarder:
            self.forwarder = Forwarder(server_settings.ctx, server_settings.pub_zmqaddr, server_settings.sub_zmqaddr)
        else:
            self.forwarder = None

    def start_threads(self):
        bokeh_app.publisher.start()
        self.subscriber.start()
        if self.forwarder:
            self.forwarder.start()

    def stop_threads(self):
        bokeh_app.publisher.stop()
        self.subscriber.stop()
        if self.forwarder:
            self.forwarder.stop()

class StaticFilter(logging.Filter):
    def filter(self, record):
        msg = record.getMessage()
        return not (msg.startswith(("200 GET /static", "200 GET /bokehjs/static")))



def make_tornado_app(flask_app=None):
    if flask_app is None:
        flask_app = app
    if server_settings.debug:
        flask_app.debug = True
    flask_app.secret_key = server_settings.secret_key
    tornado_app = SimpleBokehTornadoApp(flask_app, debug=server_settings.debug)
    tornado_app.start_threads()
    return tornado_app


# Gunicorn startup would look like
# gunicorn bokeh.server.configure.make_tornado_app(config_file=filename) -k tornado
# untested - but should work



import numpy as np

from bokeh.plotting import figure, show, output_file
from bokeh.models.sources import AjaxDataSource

output_file("ajax_source_realtime.html", title="ajax_source_realtime.py example")
source = AjaxDataSource(data_url='http://localhost:5050/data', mode="append",
                        if_modified=True, polling_interval=1000, max_size=125)
p = figure()
p.line('x', 'y', source=source)
show(p)

import time
from threading import Thread
from collections import namedtuple, deque

Entry = namedtuple('Entry', ['x', 'y', 'creation'])

entries = deque(maxlen=120)

def gen_entry():
    global entries
    x = 0
    while True:
        last_entry = Entry(x, np.sin(x*np.pi/10), time.time())
        entries.append(last_entry)
        x += 1
        if x > entries.maxlen and x % 10 == 0:
            time.sleep(2)

t = Thread(target=gen_entry)
t.daemon = True
t.start()

import json
from flask import Flask, Response, request
from bokeh.server.crossdomain import crossdomain

app = Flask(__name__)

@app.route('/data', methods=['GET', 'OPTIONS'])
@crossdomain(origin="*", methods=['GET', 'POST'])
def hello_world():
    global entries
    try:
        modified_since = float(request.headers.get('If-Modified-Since'))
    except TypeError:
        modified_since = 0

    new_entries = [e for e in entries if e.creation > modified_since]
    js = json.dumps({'x':[e.x for e in new_entries], 'y':[e.y for e in new_entries]})
    resp = Response(js, status=200, mimetype='application/json')

    if new_entries:
        resp.headers['Last-Modified'] = new_entries[-1].creation
    elif modified_since:
        resp.headers['Last-Modified'] = modified_since

    return resp

if __name__ ==  "__main__":
    app.run(port=5050)



from __future__ import absolute_import

from bokeh.plotting import curdoc, reset_output
from bokeh.session import Session
import bokeh.embed as embed

import uuid
import logging
logger = logging.getLogger(__name__)

def app_document(prefix, url="default"):
    def decorator(func):
        def wrapper(*args, **kwargs):
            reset_output()
            docname = prefix + str(uuid.uuid4())
            session = Session(name=url, root_url=url)
            session.use_doc(docname)
            session.load_document(curdoc())
            session.publish()
            curdoc().autoadd = False
            curdoc().autostore = False

            obj = func(*args, **kwargs)
            tag = embed.autoload_server(obj, session, public=True)
            obj._tag = tag

            curdoc().add(obj)
            changed = session.store_document(curdoc())

            logger.debug("stored: %s", str(changed))

            return obj
        wrapper.__name__ = func.__name__
        return wrapper
    return decorator



from bokeh.plot_object import PlotObject
from bokeh.server.utils.plugins import object_page
from bokeh.server.app import bokeh_app
from bokeh.plotting import curdoc, cursession
from bokeh.crossfilter.models import CrossFilter
from bokeh.sampledata.autompg import autompg

@bokeh_app.route("/bokeh/crossfilter/")
@object_page("crossfilter")
def make_crossfilter():
    autompg['cyl'] = autompg['cyl'].astype(str)
    autompg['origin'] = autompg['origin'].astype(str)
    app = CrossFilter.create(df=autompg)
    return app



'''
This modules exposes geometry data for Unites States. It exposes a dictionary 'data' which is
indexed by the two-tuple containing (state_id, county_id) and has the following dictionary as the
associated value:

    data[(1,1)]['name']
    data[(1,1)]['lats']
    data[(1,1)]['lons']

'''
from __future__ import absolute_import

import csv
import xml.etree.cElementTree as et
from os.path import join
from . import _data_dir

data_dir = _data_dir()

nan = float('NaN')

data = {}
with open(join(data_dir, 'US_Counties.csv')) as f:
    next(f)
    reader = csv.reader(f, delimiter=',', quotechar='"')
    for row in reader:
        name, dummy, state, dummy, geometry, dummy, dummy, dummy, dummy, state_id, county_id, dummy, dummy = row
        xml = et.fromstring(geometry)
        lats = []
        lons = []
        for i, poly in enumerate(xml.findall('.//outerBoundaryIs/LinearRing/coordinates')):
            if i > 0:
                lats.append(nan)
                lons.append(nan)
            coords = (c.split(',')[:2] for c in poly.text.split())
            lat, lon = list(zip(*[(float(lat), float(lon)) for lon, lat in
                coords]))
            lats.extend(lat)
            lons.extend(lon)
        data[(int(state_id), int(county_id))] = {
            'name' : name,
            'state' : state,
            'lats' : lats,
            'lons' : lons,
        }



""" Integrate Bokeh extensions into Sphinx autodoc.

Ensures that autodoc directives such as ``autoclass`` automatically
use Bokeh-specific directives (e.g., ``bokeh-prop`` and ``bokeh-model``)
when appropriate.

"""
from __future__ import absolute_import

from six import class_types
from sphinx.ext.autodoc import AttributeDocumenter, ClassDocumenter

from bokeh.plot_object import PlotObject
from bokeh.properties import Property

class PropDocumenter(AttributeDocumenter):
    directivetype = 'bokeh-prop'
    objtype = 'prop'
    priority = 20 # prefer over default autoattribute directive

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        # MetaHasProps creates instances even if user just
        # supplies property class name
        return isinstance(member, Property)

    def add_directive_header(self, sig):
        # Note: we are supplying our own version of this function because
        # bokeh-model should not be passed `sig` as an argument, and there is
        # no way to suppress this behaviour in the default version
        domain = getattr(self, 'domain', 'py')
        directive = getattr(self, 'directivetype', self.objtype)
        name = self.format_name()
        self.add_line(u'.. %s:%s:: %s.%s' % (domain, directive, self.modname, name),
                      '<autodoc>')
        if self.options.noindex:
            self.add_line(u'   :noindex:', '<autodoc>')

class ModelDocumenter(ClassDocumenter):
    directivetype = 'bokeh-model'
    objtype = 'model'
    priority = 20 # prefer over default autoclass directive

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, class_types) and issubclass(member, PlotObject)

    def add_directive_header(self, sig):
        # Note: we are supplying our own version of this function because
        # bokeh-model should not be passed `sig` as an argument, and there is
        # no way to suppress this behaviour in the default version
        domain = getattr(self, 'domain', 'py')
        directive = getattr(self, 'directivetype', self.objtype)
        name = self.format_name()
        self.add_line(u'.. %s:%s:: %s.%s' % (domain, directive, self.modname, name),
                      '<autodoc>')
        if self.options.noindex:
            self.add_line(u'   :noindex:', '<autodoc>')


def setup(app):
    app.add_autodocumenter(PropDocumenter)
    app.add_autodocumenter(ModelDocumenter)



from collections import OrderedDict

from bokeh._legacy_charts import Area, show, output_file

# create some example data
xyvalues = OrderedDict(
    python=[2, 3, 7, 5, 26, 221, 44, 233, 254, 265, 266, 267, 120, 111],
    pypy=[12, 33, 47, 15, 126, 121, 144, 233, 254, 225, 226, 267, 110, 130],
    jython=[22, 43, 10, 25, 26, 101, 114, 203, 194, 215, 201, 227, 139, 160],
)

output_file(filename="area.html")

area = Area(
    xyvalues, title="Responsive Area Chart",
    xlabel='time', ylabel='memory',
    stacked=True, legend="top_left",
    responsive=True
)

show(area)



# All of the other examples directly embed the Javascript and CSS code for
# Bokeh's client-side runtime into the HTML.  This leads to the HTML files
# being rather large.  An alternative is to ask Bokeh to produce HTML that
# has a relative link to the Bokeh Javascript and CSS.  This is easy to
# do; you just pass in a few extra arguments to the output_file() command.

import numpy as np

from bokeh.plotting import figure, show, output_file

N = 100
x = np.linspace(0, 4*np.pi, N)
y = np.sin(x)

output_file("relative_paths.html", title="Relative path example", mode="relative")

p = figure(tools="pan,wheel_zoom,box_zoom,reset,save")
p.circle(x,y, alpha=0.5, color="tomato", radius=0.1)

show(p)

# By default, the URLs for the Javascript and CSS will be relative to
# the current directory, i.e. the directory in which the HTML file is
# generated.  You can provide a different "root" directory from which
# the relative paths will be computed:
#
# output_file("relative_paths.html", title="Relative path example",
#             resources="relative", rootdir="some/other/path")



''' Define standard error codes and messages for Bokeh validation checks.

1000 : *COLUMN_LENGTHS*
    A |ColumnDataSource| has columns whose lengths are not all the same.

1001 : *BAD_COLUMN_NAME*
    A glyph has a property set to a field name that does not correspond to any
    column in the |GlyphRenderer|'s data source.

1002 : *MISSING_GLYPH*
    A |GlyphRenderer| has no glyph configured.

1003 : *NO_SOURCE_FOR_GLYPH*
    A |GlyphRenderer| has no data source configured.

1004 : *REQUIRED_RANGE*
    A |Plot| is missing one or more required default ranges (will result in
    blank plot).

9999 : *EXT*
    Indicates that a custom error check has failed.

'''

codes = {
    1000: ("COLUMN_LENGTHS",        "ColumnDataSource column lengths are not all the same"),
    1001: ("BAD_COLUMN_NAME",       "Glyph refers to nonexistent column name"),
    1002: ("MISSING_GLYPH",         "Glyph renderer has no glyph set"),
    1003: ("NO_SOURCE_FOR_GLYPH",   "Glyph renderer has no data source"),
    1004: ("REQUIRED_RANGE",        "A required Range objects missing"),
    9999: ("EXT",                   "Custom extension reports error"),
}

for code in codes:
    exec("%s = %d" % (codes[code][0], code))







from __future__ import print_function

from bokeh.browserlib import view
from bokeh.document import Document
from bokeh.embed import file_html
from bokeh.models.glyphs import Circle
from bokeh.models import (
    Plot, DataRange1d, LinearAxis, ColumnDataSource, PanTool, WheelZoomTool
)
from bokeh.resources import INLINE

source = ColumnDataSource(
    data = dict(
        x = [1, 2, 3, 4, 4,   5, 5],
        y = [5, 4, 3, 2, 2.1, 1, 1.1],
        color = ["rgb(0, 100, 120)", "green", "blue", "#2c7fb8", "#2c7fb8", "rgba(120, 230, 150, 0.5)", "rgba(120, 230, 150, 0.5)"]
    )
)

xdr = DataRange1d()
ydr = DataRange1d()

plot = Plot(x_range=xdr, y_range=ydr)

circle = Circle(x="x", y="y", radius=0.2,
    # Set the fill color to be dependent on the "color" field of the
    # data source.  If the field is missing, then the default value is
    # used. Since no explicit default is provided, this picks up the
    # default in FillProps, which is "gray".
    fill_color="color",

    # Alternative to using fill_color with rgba values, you can also use
    # the fill_alpha setting to set the alpha values of your circle or
    # other glyphs. This can be set as a single value or powered by a
    # column in your data source. Uncomment the following line
    # to see the effect.
    # fill_alpha=0.2,

    # An alternative form that explicitly sets a default value:
    #fill_color={"default": "red", "field": "color"},

    # Note that line_color is set to a fixed value. This can be any of
    # the SVG named 147 colors, or a hex color string starting with "#",
    # or a string "rgb(r,g,b)" or "rgba(r,g,b,a)".
    # Any other string will be interpreted as a field name to look up
    # on the datasource.
    line_color="black")

plot.add_glyph(source, circle)

plot.add_layout(LinearAxis(), 'below')
plot.add_layout(LinearAxis(), 'left')

plot.add_tools(PanTool(), WheelZoomTool())

doc = Document()
doc.add(plot)

if __name__ == "__main__":
    filename = "colorspec.html"
    with open(filename, "w") as f:
        f.write(file_html(doc, INLINE, "Demonstration of ColorSpec"))
    print("Wrote %s" % filename)
    view(filename)



from __future__ import absolute_import
from os.path import dirname, join

try:
    import pandas as pd
except ImportError as e:
    raise RuntimeError("mtb data requires pandas (http://pandas.pydata.org) to be installed")

obiszow_mtb_xcm = pd.read_csv(join(dirname(__file__), 'obiszow_mtb_xcm.csv'))



# coding: utf-8

# Plotting U.S. marriage and divorce statistics
#
# Example code by Randal S. Olson (http://www.randalolson.com)

from bokeh.plotting import figure, show, output_file, ColumnDataSource
from bokeh.models import HoverTool, NumeralTickFormatter
from bokeh.models import SingleIntervalTicker, LinearAxis
import pandas as pd

# Since the data set is loaded in the bokeh data repository, we can do this:

from bokeh.sampledata.us_marriages_divorces import data

md_data = data.copy()

# Fill in missing data with a simple linear interpolation
md_data = md_data.interpolate(method='linear', axis=0).ffill().bfill()

# Tell Bokeh where to save the interactive chart
output_file('us_marriages_divorces_per_capita.html',
            # Tell Bokeh to use its minified JavaScript hosted on a
            # cdn instead of putting the Bokeh JS in the output file
            # Warning: This makes it so people can only view the
            # chart with an internet connection
            mode='cdn',
            title='144 years of marriage and divorce in the U.S.A.')

# Set up the data sources for the lines we'll be plotting.
# We need separate data sources for each line because we're
# displaying different data in the hover tool.

source_marriages = ColumnDataSource(
    data=dict(
        # x-axis (Years) for the chart
        x=md_data.Year.values,
        # y-axis (Marriages per capita) for the chart
        y=md_data.Marriages_per_1000.values,
        # The string version of the y-value that is displayed in the hover box
        y_text=md_data.Marriages_per_1000.apply(
            lambda x: '{}'.format(round(x, 1))),
        # Extra descriptive text that is displayed in the hover box
        desc=['marriages per 1,000 people'] * len(md_data),
    )
)

source_divorces = ColumnDataSource(
    data=dict(
        # x-axis (Years) for the chart
        x=md_data.Year.values,
        # y-axis (Marriages per capita) for the chart
        y=md_data.Divorces_per_1000.values,
        # The string version of the y-value that is displayed in the hover box
        y_text=md_data.Divorces_per_1000.apply(
            lambda x: '{}'.format(round(x, 1))),
        # Extra descriptive text that is displayed in the hover box
        desc=['divorces and annulments per 1,000 people'] * len(md_data),
    )
)

# Use HTML to mark up the tooltip that displays over the chart
# Note that the variables in the data sources (above) are referenced with a @
hover = HoverTool(
    tooltips='<font face="Arial" size="3">@y_text @desc in @x</font>')

# Select the tools that will be available to the chart
TOOLS = ['pan,wheel_zoom,box_zoom,reset,save,resize'] + [hover]

bplot = figure(tools=TOOLS, width=800, height=500, x_axis_type=None)

# Create a custom x-axis with 10-year intervals
ticker = SingleIntervalTicker(interval=10, num_minor_ticks=0)
xaxis = LinearAxis(ticker=ticker)
bplot.add_layout(xaxis, 'below')

# Customize the y-axis
bplot.yaxis.formatter = NumeralTickFormatter(format='0.0a')
bplot.yaxis.axis_label = '# per 1,000 people'

# Provide a descriptive title for the chart
bplot.title = '144 years of marriage and divorce in the U.S.'

# Finally, plot the data!
# Note that the data source determines what is plotted and what shows in
# the tooltips
bplot.line('x', 'y', color='#1f77b4', line_width=3, source=source_marriages)
bplot.line('x', 'y', color='#ff7f0e', line_width=3, source=source_divorces)

show(bplot)



from os.path import join, dirname
import time

from bokeh.server.settings import settings as server_settings
from bokeh.server.forwarder import Forwarder

if __name__ == "__main__":
    config_file = join(dirname(__file__), 'config.py')
    server_settings.from_file(config_file)
    forwarder = Forwarder(server_settings.ctx,
                          server_settings.pub_zmqaddr,
                          server_settings.sub_zmqaddr)
    try:
        forwarder.start()
        while True:
            time.sleep(10)
    except KeyboardInterrupt:
        forwarder.stop()



# Example similar to line.py, but demoing special data
# like masked arrays, nans, and inf

import numpy as np

from bokeh.plotting import figure, output_file, show

x = np.linspace(0, 4*np.pi, 200)
y1 = np.sin(x)
y2 = np.cos(x)

# Set high/low values to inf
y1[y1>+0.9] = +np.inf
y1[y1<-0.9] = -np.inf

# Set high values to nan and mask the low
y2[y2>0.9] = np.nan
y2 = np.ma.masked_array(y2, y2<-0.9)

output_file("line_missing_data.html", title="line_missing_data.py example")

p = figure(title="lines with some missing/inf values")
p.line(x,y1, color="#2222aa", line_width=2)
p.line(x,y2, color="#22aa22", line_width=2)

show(p)



''' Provide the ``check_integrity`` function.

'''
from __future__ import absolute_import

import logging
logger = logging.getLogger(__file__)


def check_integrity(models):

    messages = dict(error=[], warning=[])

    for model in models:
        validators = []
        for name in dir(model):
            if not name.startswith("_check"): continue
            obj = getattr(model, name)
            if getattr(obj, "validator_type", None):
                validators.append(obj)
        for func in validators:
            messages[func.validator_type].extend(func())

    for msg in sorted(messages['error']):
        logger.error("E-%d (%s): %s: %s" % msg)

    for msg in sorted(messages['warning']):
        logger.error("W-%d (%s): %s: %s" % msg)

    # This will be turned on in a future release
    # if len(messages['error']) or (len(messages['warning']) and settings.strict()):
    #     raise ValidationError("Errors encountered during validation (see log output)")



""" This is the Bokeh charts testing interface.

"""
#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2014, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------
from __future__ import absolute_import

from collections import OrderedDict
import unittest

import numpy as np
from numpy.testing import assert_array_equal
import pandas as pd

from bokeh._legacy_charts import Line

from ._utils import create_chart

#-----------------------------------------------------------------------------
# Classes and functions
#-----------------------------------------------------------------------------

class TestLine(unittest.TestCase):
    def test_supported_input(self):
        xyvalues = OrderedDict()
        y_python = xyvalues['python'] = [2, 3, 7, 5, 26]
        y_pypy = xyvalues['pypy'] = [12, 33, 47, 15, 126]
        y_jython = xyvalues['jython'] = [22, 43, 10, 25, 26]

        xyvaluesdf = pd.DataFrame(xyvalues)

        for i, _xy in enumerate([xyvalues, xyvaluesdf]):
            hm = create_chart(Line, _xy)
            builder = hm._builders[0]
            self.assertEqual(sorted(builder._groups), sorted(list(xyvalues.keys())))
            assert_array_equal(builder._data['x'], [0, 1, 2, 3, 4])
            assert_array_equal(builder._data['y_python'], y_python)
            assert_array_equal(builder._data['y_pypy'], y_pypy)
            assert_array_equal(builder._data['y_jython'], y_jython)

        lvalues = [[2, 3, 7, 5, 26], [12, 33, 47, 15, 126], [22, 43, 10, 25, 26]]
        for _xy in [lvalues, np.array(lvalues)]:
            hm = create_chart(Line, _xy)
            builder = hm._builders[0]
            self.assertEqual(builder._groups, ['0', '1', '2'])
            assert_array_equal(builder._data['x'], [0, 1, 2, 3, 4])
            assert_array_equal(builder._data['y_0'], y_python)
            assert_array_equal(builder._data['y_1'], y_pypy)
            assert_array_equal(builder._data['y_2'], y_jython)



#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2015, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
from __future__ import absolute_import

import flask

from ..app import bokeh_app

## This URL heirarchy is important, because of the way we build bokehjs
## the source mappings list the source file as being inside ../../src

@bokeh_app.route('/bokehjs/static/<path:filename>')
def bokehjs_file(filename):
    """ Return a specific BokehJS deployment file

    :param filename: name of the file to retrieve

    :status 200: file is found
    :status 404: file is not found

    """
    return flask.send_from_directory(bokeh_app.bokehjsdir, filename)

@bokeh_app.route('/bokehjs/src/<path:filename>')
def bokehjssrc_file(filename):
    """ Return a specific BokehJS source code file

    :param filename: name of the file to retrieve

    :status 200: file is found
    :status 404: file is not found

    """
    return flask.send_from_directory(bokeh_app.bokehjssrcdir, filename)



#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2015, Continuum Analytics, Inc. All rights reserved.
#
# Powered by the Bokeh Development Team.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
from __future__ import absolute_import

import logging
logger = logging.getLogger(__name__)

from bokeh.exceptions import AuthenticationException

from .app import bokeh_app

def prune(document, temporary_docid=None, delete=False):
    if temporary_docid is not None:
        storage_id = temporary_docid
    else:
        storage_id = document.docid
    to_delete = document.prune()
    if delete:
        for obj in to_delete:
            bokeh_app.backbone_storage.del_obj(storage_id, obj)

def get_temporary_docid(request, docid):
    key = 'temporary-%s' % docid
    return request.headers.get(key, None)

class BokehServerTransaction(object):
    #hugo - is this the right name?
    """Context Manager for a req/rep response cycle of the bokeh server
    responsible for
    -  determining whether the current user can read from a document
    -  determining whether the current user can write to a bokeh document
    (or temporary document for copy on write)
    -  stitching together documents to form a copy on write view
    -  saving changes to the document

    at the start of the context manager, self.clientdoc is populated
    with an instance of bokeh.document.Document with all the data
    loaded in (including from the copy on write context if specified)
    at the end of the context manager, changed models are written
    to the appropriate storage location.  and changed models are
    written to self.changed

    currently deletes aren't really working properly with cow - but we
    don't really make use of model deletions yet
    """
    def __init__(self, server_userobj, server_docobj, mode,
                 temporary_docid=None):
        """
        bokeh_app : bokeh_app blueprint
        server_userobj : instance of bokeh.server.models.user.User - current user
          for a request
        server_docobj : instance of bokeh.server.models.docs.Doc
        mode : 'r', or 'rw', or 'auto' - auto means rw if possible, else r
        temporary_docid : temporary docid for copy on write
        """
        logger.debug(
            "created transaction with %s, %s",
            server_docobj.docid, temporary_docid
        )
        self.server_userobj = server_userobj
        self.server_docobj = server_docobj
        self.temporary_docid = temporary_docid
        can_write = bokeh_app.authentication.can_write_doc(
            self.server_docobj,
            userobj=self.server_userobj,
            temporary_docid=self.temporary_docid)
        if can_write:
            can_read = True
        else:
            can_read = bokeh_app.authentication.can_read_doc(
                self.server_docobj,
                userobj=self.server_userobj)
        docid = self.server_docobj.docid
        if mode not in {'auto', 'rw', 'r'}:
            raise AuthenticationException('Unknown Mode')
        if mode == 'auto':
            if not can_write and not can_read:
                raise AuthenticationException("could not read from %s" % docid)
            if can_write:
                mode = 'rw'
            else:
                mode = 'r'
        else:
            if mode == 'rw':
                if not can_write:
                    raise AuthenticationException("could not write to %s" % docid)
            elif mode == 'r':
                if not can_read:
                    raise AuthenticationException("could not read from %s" % docid)
        self.mode = mode
        if self.mode == 'rw':
            self.apikey = self.server_docobj.apikey
        else:
            self.apikey = self.server_docobj.readonlyapikey
    @property
    def write_docid(self):
        if self.temporary_docid:
            return self.temporary_docid
        else:
            return self.server_docobj.docid

    def load(self, gc=False):
        from .views.backbone import init_bokeh
        clientdoc = bokeh_app.backbone_storage.get_document(self.server_docobj.docid)
        if self.temporary_docid:
            temporary_json = bokeh_app.backbone_storage.pull(self.temporary_docid)
            #no events - because we're loading from datastore, so nothing is new
            clientdoc.load(*temporary_json, events='none', dirty=False)
        if gc and self.mode != 'rw':
            raise AuthenticationException("cannot run garbage collection in read only mode")
        elif gc and self.mode == 'rw':
            prune(clientdoc, delete=True)
        else:
            prune(clientdoc)
        init_bokeh(clientdoc)
        self.clientdoc = clientdoc

    def save(self):
        if self.mode != 'rw':
            raise AuthenticationException("cannot save in read only mode")
        self.changed = bokeh_app.backbone_storage.store_document(
            self.clientdoc,
            temporary_docid=self.temporary_docid
        )
        logger.debug("stored %s models", len(self.changed))



'''
This modules exposes geometry data for Unites States. It exposes a dictionary 'data' which is
indexed by the two letter state code (e.g., 'CA', 'TX') and has the following dictionary as the
associated value:

    data['CA']['name']
    data['CA']['region']
    data['CA']['lats']
    data['CA']['lons']

'''
from __future__ import absolute_import

import csv
import codecs
import gzip
import xml.etree.cElementTree as et
from os.path import dirname, join

nan = float('NaN')

data = {}
with gzip.open(join(dirname(__file__), 'US_Regions_State_Boundaries.csv.gz')) as f:
    decoded = codecs.iterdecode(f, "utf-8")
    next(decoded)
    reader = csv.reader(decoded, delimiter=',', quotechar='"')
    for row in reader:
        region, name, code, geometry, dummy = row
        xml = et.fromstring(geometry)
        lats = []
        lons = []
        for i, poly in enumerate(xml.findall('.//outerBoundaryIs/LinearRing/coordinates')):
            if i > 0:
                lats.append(nan)
                lons.append(nan)
            coords = (c.split(',')[:2] for c in poly.text.split())
            lat, lon = list(zip(*[(float(lat), float(lon)) for lon, lat in
                coords]))
            lats.extend(lat)
            lons.extend(lon)
        data[code] = {
            'name'   : name,
            'region' : region,
            'lats'   : lats,
            'lons'   : lons,
        }



""" Classes that can be mixed-in to Bokeh model classes to add sets of
related properties in bulk. """

from __future__ import absolute_import

from .properties import (
    HasProps, ColorSpec, Enum, DashPattern, Int, NumberSpec, String, FontSizeSpec)
from .enums import LineJoin, LineCap, FontStyle, TextAlign, TextBaseline

class FillProps(HasProps):
    """ Properties to use when performing fill operations while rendering.

    Mirrors the BokehJS ``properties.Fill`` class.

    """

    fill_color = ColorSpec(default="gray", help="""
    A color to use to fill paths with.

    Acceptable values are:

    - any of the 147 named `CSS colors`_, e.g ``'green'``, ``'indigo'``
    - an RGB(A) hex value, e.g., ``'#FF0000'``, ``'#44444444'``
    - a 3-tuple of integers (r,g,b) between 0 and 255
    - a 4-tuple of (r,g,b,a) where r,g,b are integers between 0..255 and a is between 0..1

    .. _CSS colors: http://www.w3schools.com/cssref/css_colornames.asp

    """)

    fill_alpha = NumberSpec(default=1.0, help="""
    An alpha value to use to fill paths with.

    Acceptable values are floating point numbers between 0 (transparent)
    and 1 (opaque).

    """)

class LineProps(HasProps):
    """ Properties to use when performing stroke operations while rendering.

    Mirrors the BokehJS ``properties.Line`` class.

    """

    line_color = ColorSpec(default="black", help="""
    A color to use to stroke paths with.

    Acceptable values are:

    - any of the 147 named `CSS colors`_, e.g ``'green'``, ``'indigo'``
    - an RGB(A) hex value, e.g., ``'#FF0000'``, ``'#44444444'``
    - a 3-tuple of integers (r,g,b) between 0 and 255
    - a 4-tuple of (r,g,b,a) where r,g,b are integers between 0..255 and a is between 0..1

    .. _CSS colors: http://www.w3schools.com/cssref/css_colornames.asp

    """)

    line_width = NumberSpec(default=1, help="""
    Stroke width in units of pixels.
    """)

    line_alpha = NumberSpec(default=1.0, help="""
    An alpha value to use to stroke paths with.

    Acceptable values are floating point numbers between 0 (transparent)
    and 1 (opaque).

    """)

    line_join = Enum(LineJoin, help="""
    How path segments should be joined together.

    Acceptable values are:

    - ``'miter'`` |miter_join|
    - ``'round'`` |round_join|
    - ``'bevel'`` |bevel_join|

    .. |miter_join| image:: /_images/miter_join.png
       :height: 20
    .. |round_join| image:: /_images/round_join.png
       :height: 20
    .. |bevel_join| image:: /_images/bevel_join.png
       :height: 20

    """)

    line_cap = Enum(LineCap, help="""
    How path segments should be terminated.

    Acceptable values are:

    - ``'butt'`` |butt_cap|
    - ``'round'`` |round_cap|
    - ``'square'`` |square_cap|

    .. |butt_cap| image:: /_images/butt_cap.png
       :height: 20
    .. |round_cap| image:: /_images/round_cap.png
       :height: 20
    .. |square_cap| image:: /_images/square_cap.png
       :height: 20

    """)

    line_dash = DashPattern(help="""
    How should the line be dashed.
    """)

    line_dash_offset = Int(0, help="""
    The distance into the ``line_dash`` (in pixels) that the pattern should
    start from.
    """)

class TextProps(HasProps):
    """ Properties to use when performing text drawing operations while
    rendering.

    Mirrors the BokehJS ``properties.Text`` class.

    .. note::
        There is currently only support for filling text. An interface
        to stroke the outlines of text has not yet been exposed.

    """

    text_font = String("Helvetica", help="""
    Name of a font to use for rendering text, e.g., ``'times'``,
    ``'helvetica'``.

    """)

    text_font_size = FontSizeSpec("12pt")

    text_font_style = Enum(FontStyle, help="""
    A style to use for rendering text.

    Acceptable values are:

    - ``'normal'`` normal text
    - ``'italic'`` *italic text*
    - ``'bold'`` **bold text**

    """)

    text_color = ColorSpec(default="#444444", help="""
    A color to use to fill text with.

    Acceptable values are:

    - any of the 147 named `CSS colors`_, e.g ``'green'``, ``'indigo'``
    - an RGB(A) hex value, e.g., ``'#FF0000'``, ``'#44444444'``
    - a 3-tuple of integers (r,g,b) between 0 and 255
    - a 4-tuple of (r,g,b,a) where r,g,b are integers between 0..255 and a is between 0..1

    .. _CSS colors: http://www.w3schools.com/cssref/css_colornames.asp

    """)

    text_alpha = NumberSpec(default=1.0, help="""
    An alpha value to use to fill text with.

    Acceptable values are floating point numbers between 0 (transparent)
    and 1 (opaque).

    """)

    text_align = Enum(TextAlign, help="""
    Horizontal anchor point to use when rendering text.

    Acceptable values are:

    - ``'left'``
    - ``'right'``
    - ``'center'``

    """)

    text_baseline = Enum(TextBaseline, default="bottom", help="""
    Vertical anchor point to use when rendering text.

    Acceptable values are:

    - ``'top'``
    - ``'middle'``
    - ``'bottom'``
    - ``'alphabetic'``
    - ``'hanging'``

    """)



from bokeh._legacy_charts import HeatMap, output_file, show
from bokeh.palettes import YlOrRd9 as palette
from bokeh.sampledata.unemployment1948 import data

# pandas magic
df = data[data.columns[:-1]]
df2 = df.set_index(df[df.columns[0]].astype(str))
df2.drop(df.columns[0], axis=1, inplace=True)
df3 = df2.transpose()

output_file("cat_heatmap.html")

palette = palette[::-1]  # Reverse the color order so dark red is highest unemployment
hm = HeatMap(df3, title="categorical heatmap", width=800, palette=palette)

show(hm)



""" Generate color representations of all known Bokeh palettes.

Usage
-----

This directive takes no arguments.

"""
from __future__ import absolute_import

from docutils import nodes

import jinja2

from sphinx.locale import _
from sphinx.util.compat import Directive

from bokeh.palettes import brewer

BREWER_TEMPLATE = jinja2.Template(u"""
<table>

  <tr>
    <th> {{ name }} </th>
  </tr>

  {% for number in numbers %}
  <tr>

    <td height='20px' width='30px'> {{ number }} </td>

    {% for color in palette[number] %}
    <td height="20px" width="20px" style="background-color: {{ color }};"/>
    {% endfor %}

  </tr>
  {% endfor %}

</table>
""")


class bokeh_palette(nodes.General, nodes.Element):
    pass


class BokehPaletteDirective(Directive):

    has_content = False
    required_arguments = 1

    def run(self):
        node = bokeh_palette()
        node['module'] = self.arguments[0]
        return [node]

def html_visit_bokeh_palette(self, node):
    # NOTE: currently only handles the existing Brewer palettes
    names = sorted(brewer)
    for name in names:
        palette = brewer[name]
        numbers = sorted(palette)

        html = BREWER_TEMPLATE.render(name=name, numbers=numbers, palette=palette)
        self.body.append(html)
    raise nodes.SkipNode

def latex_visit_bokeh_palette(self, node):
    self.body.append(_('[palette: %s]' % node['module']))
    raise nodes.SkipNode


def texinfo_visit_bokeh_palette(self, node):
    self.body.append(_('[palette: %s]' % node['module']))
    raise nodes.SkipNode


def text_visit_bokeh_palette(self, node):
    self.body.append(_('[palette: %s]' % node['module']))
    raise nodes.SkipNode


def man_visit_bokeh_palette(self, node):
    self.body.append(_('[palette: %s]' % node['module']))
    raise nodes.SkipNode


def setup(app):
    app.add_node(bokeh_palette,
                 html=(html_visit_bokeh_palette, None),
                 latex=(latex_visit_bokeh_palette, None),
                 texinfo=(texinfo_visit_bokeh_palette, None),
                 text=(text_visit_bokeh_palette, None),
                 man=(man_visit_bokeh_palette, None))
    app.add_directive('bokeh-palette', BokehPaletteDirective)









from __future__ import absolute_import

import unittest

import bokeh.query as query

from bokeh.models import (
    Axis, BoxZoomTool, ColumnDataSource, DatetimeAxis, GlyphRenderer, Grid, LinearAxis,
    LogAxis, PanTool, Plot, PreviewSaveTool, Range1d, ResetTool, ResizeTool, Tool, WheelZoomTool,
)
from bokeh.models.glyphs import Glyph, Circle, Line, Rect

def large_plot():
    source = ColumnDataSource(data=dict(x=[0, 1], y=[0, 1]))

    xdr = Range1d(start=0, end=1)
    xdr.tags.append("foo")
    xdr.tags.append("bar")

    ydr = Range1d(start=10, end=20)
    ydr.tags.append("foo")

    plot = Plot(x_range=xdr, y_range=ydr)

    ydr2 = Range1d(start=0, end=100)
    plot.extra_y_ranges = {"liny": ydr2}

    circle = Circle(x="x", y="y", fill_color="red", size=5, line_color="black")
    plot.add_glyph(source, circle, name="mycircle")

    line = Line(x="x", y="y")
    plot.add_glyph(source, line, name="myline")

    rect = Rect(x="x", y="y", width=1, height=1, fill_color="green")
    plot.add_glyph(source, rect, name="myrect")

    plot.add_layout(DatetimeAxis(), 'below')
    plot.add_layout(LogAxis(), 'left')
    plot.add_layout(LinearAxis(y_range_name="liny"), 'left')

    plot.add_layout(Grid(dimension=0), 'left')
    plot.add_layout(Grid(dimension=1), 'left')

    plot.add_tools(
        BoxZoomTool(), PanTool(), PreviewSaveTool(), ResetTool(), ResizeTool(), WheelZoomTool(),
    )

    return plot

class TestMatch(unittest.TestCase):

    def test_type(self):
        pass

typcases = {
    Range1d: 3,

    Plot: 1,
    Glyph: 3,

    Axis: 3,
    DatetimeAxis: 1,
    LinearAxis: 2,  # DatetimeAxis is subclass of LinearAxis
    LogAxis: 1,

    Grid: 2,

    Tool: 6,
    BoxZoomTool: 1,
    PanTool: 1,
    PreviewSaveTool: 1,
    ResetTool: 1,
    ResizeTool: 1,
    WheelZoomTool: 1,
}

class TestFind(unittest.TestCase):

    def setUp(self):
        self.plot = large_plot()

    def test_type(self):

        for typ, count in typcases.items():
            res = list(query.find(self.plot.references(), dict(type=typ)))
            self.assertEqual(len(res), count)
            self.assertTrue(all(isinstance(x, typ) for x in res))

    def test_tags_with_string(self):
        cases = {
            "foo": 2,
            "bar": 1,
        }

        for tag, count in cases.items():
            res = list(query.find(self.plot.references(), dict(tags=tag)))
            self.assertEqual(len(res), count)

    def test_tags_with_seq(self):
        cases = {
            "foo": 2,
            "bar": 1,
        }

        for tag, count in cases.items():
            res = list(query.find(self.plot.references(), dict(tags=[tag])))
            self.assertEqual(len(res), count)

        res = list(query.find(self.plot.references(), dict(tags=list(cases.keys()))))
        self.assertEqual(len(res), 2)

    def test_name(self):
        cases = {
            "myline": Line,
            "mycircle": Circle,
            "myrect": Rect,
        }

        for name, typ in cases.items():
            res = list(query.find(self.plot.references(), dict(name=name)))
            self.assertEqual(len(res), 1)
            self.assertTrue(all(isinstance(x.glyph, typ) for x in res))

    def test_in(self):
        from bokeh.query import IN

        res = list(query.find(self.plot.references(), dict(name={IN: ['a', 'b']})))
        self.assertEqual(len(res), 0)

        res = list(query.find(self.plot.references(), dict(name={IN: ['a', 'mycircle']})))
        self.assertEqual(len(res), 1)

        res = list(query.find(self.plot.references(), dict(name={IN: ['a', 'mycircle', 'myline']})))
        self.assertEqual(len(res), 2)

        res = list(query.find(self.plot.references(), dict(name={IN: ['a', 'mycircle', 'myline', 'myrect']})))
        self.assertEqual(len(res), 3)

        for typ, count in typcases.items():
            res = list(query.find(self.plot.references(), dict(type={IN: [typ]})))
            self.assertEqual(len(res), count)
            self.assertTrue(all(isinstance(x, typ) for x in res))

            res = list(query.find(self.plot.references(), dict(type={IN: [typ, dict]})))
            self.assertEqual(len(res), count)
            self.assertTrue(all(isinstance(x, typ) for x in res))

            res = list(query.find(self.plot.references(), dict(type={IN: [dict]})))
            self.assertEqual(len(res), 0)

        # count adjusted by hand to account for duplicates/subclasses
        res = list(query.find(self.plot.references(), dict(type={IN: list(typcases.keys())})))
        self.assertEqual(len(res), 18)


    def test_disjuction(self):
        from bokeh.query import OR

        res = list(
            query.find(self.plot.references(),
            {OR: [dict(type=Axis), dict(type=Grid)]})
        )
        self.assertEqual(len(res), 5)

        res = list(
            query.find(self.plot.references(),
            {OR: [dict(type=Axis), dict(name="mycircle")]})
        )
        self.assertEqual(len(res), 4)

        res = list(
            query.find(self.plot.references(),
            {OR: [dict(type=Axis), dict(tags="foo"), dict(name="mycircle")]})
        )
        self.assertEqual(len(res), 6)

        res = list(
            query.find(self.plot.references(),
            {OR: [dict(type=Axis), dict(tags="foo"), dict(name="mycircle"), dict(name="bad")]})
        )
        self.assertEqual(len(res), 6)

    def test_conjuction(self):
        res = list(
            query.find(self.plot.references(), dict(type=Axis, tags="foo"))
        )
        self.assertEqual(len(res), 0)

        res = list(
            query.find(self.plot.references(), dict(type=Range1d, tags="foo"))
        )
        self.assertEqual(len(res), 2)

        res = list(
            query.find(self.plot.references(), dict(type=GlyphRenderer, name="mycircle"))
        )
        self.assertEqual(len(res), 1)

    def test_ops(self):
        from bokeh.query import EQ, LEQ, GEQ, LT, GT, NEQ

        res = list(
            query.find(self.plot.references(), {'size': {EQ: 5}})
        )
        self.assertEqual(len(res), 1)

        res = list(
            query.find(self.plot.references(), {'size': {NEQ: 5}})
        )
        self.assertEqual(len(res), 0)

        res = list(
            query.find(self.plot.references(), {'size': {GEQ: 5}})
        )
        self.assertEqual(len(res), 1)

        res = list(
            query.find(self.plot.references(), {'size': {LEQ: 5}})
        )
        self.assertEqual(len(res), 1)

        res = list(
            query.find(self.plot.references(), {'size': {GT: 5}})
        )
        self.assertEqual(len(res), 0)

        res = list(
            query.find(self.plot.references(), {'size': {LT: 5}})
        )
        self.assertEqual(len(res), 0)

if __name__ == "__main__":
    unittest.main()



from bokeh.plotting import figure, show, output_file

output_file("image_url.html")

p = figure()

url = ["http://bokeh.pydata.org/en/latest/_static/bokeh-transparent.png"]*10
x = list(range(0, 100, 10))
y = list(range(0, 100, 10))
p.image_url(x=x, y=y, url=url, global_alpha=0.2)

show(p)



from __future__ import absolute_import

try:
    import pandas as pd
except ImportError as e:
    raise RuntimeError("population data requires pandas (http://pandas.pydata.org) to be installed")

from . import _data_dir

def load_population():
    csv_file = _data_dir("WPP2012_SA_DB03_POPULATION_QUINQUENNIAL.csv")
    df = pd.read_csv(csv_file, encoding="CP1250")
    df = df[df.Sex != "Both"]
    df = df.drop(["VarID", "Variant", "MidPeriod", "SexID", "AgeGrpSpan"], axis=1)
    df = df.rename(columns={"Time": "Year"})
    df.Value *= 1000
    return df



''' The session module provides the Session class, which encapsulates a
connection to a Document that resides on a Bokeh server.

The Session class provides methods for creating, loading and storing
documents and objects, as well as methods for user-authentication. These
are useful when the server is run in multi-user mode.

'''
from __future__ import absolute_import, print_function

#--------
# logging
#--------
import logging
logger = logging.getLogger(__name__)

#-------------
# standard lib
#-------------
import time
import json
from os import makedirs
from os.path import expanduser, exists, join
import tempfile

#------------
# third party
#------------
from six.moves.urllib.parse import urlencode
from requests.exceptions import ConnectionError

#---------
# optional
#---------
try:
    import pandas as pd
    import tables
    has_pandas = True
except ImportError as e:
    has_pandas = False

#--------
# project
#--------
from . import browserlib
from . import protocol
from .embed import autoload_server
from .exceptions import DataIntegrityException
from .util.notebook import publish_display_data
from .util.serialization import dump, get_json, urljoin

DEFAULT_SERVER_URL = "http://localhost:5006/"

class Session(object):
    """ Encapsulate a connection to a document stored on a Bokeh Server.

    Args:
        name (str, optional) : name of server
        root_url (str, optional) : root url of server
        userapikey (str, optional) : (default: "nokey")
        username (str, optional) : (default: "defaultuser")
        load_from_config (bool, optional) :
            Whether to load login information from config. (default: True)
            If False, then we may overwrite the user's config.
        configdir (str) : location of user configuration information

    Attributes:
        base_url (str) :
        configdir (str) :
        configfile (str) :
        http_session (requests.session) :
        userapikey (str) :
        userinfo (dict) :
        username (str) :

    """
    def __init__(
            self,
            name             = DEFAULT_SERVER_URL,
            root_url         = DEFAULT_SERVER_URL,
            userapikey       = "nokey",
            username         = "defaultuser",
            load_from_config = True,
            configdir        = None,
        ):

        self.name = name

        if not root_url.endswith("/"):
            logger.warning("root_url should end with a /, adding one")
            root_url = root_url + "/"
        self.root_url = root_url

        # single user mode case
        self.userapikey = userapikey
        self.username = username
        self._configdir = None

        if configdir:
            self.configdir = configdir

        if load_from_config:
            self.load()

    @property
    def http_session(self):
        if hasattr(self, "_http_session"):
            return self._http_session
        else:
            import requests
            self._http_session = requests.session()
            return self._http_session

    @property
    def username(self):
        return self.http_session.headers.get('BOKEHUSER')

    @username.setter
    def username(self, val):
        self.http_session.headers.update({'BOKEHUSER': val})

    @property
    def userapikey(self):
        return self.http_session.headers.get('BOKEHUSER-API-KEY')

    @userapikey.setter
    def userapikey(self, val):
        self.http_session.headers.update({'BOKEHUSER-API-KEY': val})

    @property
    def configdir(self):
        """ filename where our config are stored. """
        if self._configdir:
            return self._configdir
        bokehdir = join(expanduser("~"), ".bokeh")
        if not exists(bokehdir):
            makedirs(bokehdir)
        return bokehdir

    # for testing
    @configdir.setter
    def configdir(self, path):
        self._configdir = path

    @property
    def configfile(self):
        return join(self.configdir, "config.json")

    def load_dict(self):
        configfile = self.configfile
        if not exists(configfile):
            data = {}
        else:
            with open(configfile, "r") as f:
                data = json.load(f)
        return data

    def load(self):
        """ Loads the server configuration information from disk

        Returns:
            None

        """
        config_info = self.load_dict().get(self.name, {})
        print("Using saved session configuration for %s" % self.name)
        print("To override, pass 'load_from_config=False' to Session")
        self.root_url = config_info.get('root_url', self.root_url)
        self.userapikey = config_info.get('userapikey', self.userapikey)
        self.username = config_info.get('username', self.username)

    def save(self):
        """ Save the server configuration information to JSON

        Returns:
            None

        """
        data = self.load_dict()
        data[self.name] = {'root_url': self.root_url,
                           'userapikey': self.userapikey,
                           'username': self.username}
        configfile = self.configfile
        with open(configfile, "w+") as f:
            json.dump(data, f)

    def register(self, username, password):
        ''' Register a new user with a bokeh server.

        .. note::
            This is useful in multi-user mode.

        Args:
            username (str) : user name to register
            password (str) : user password for account

        Returns:
            None

        '''
        url = urljoin(self.root_url, "bokeh/register")
        result = self.execute('post', url, data={
            'username': username,
            'password': password,
            'api': 'true'
        })
        if result.status_code != 200:
            raise RuntimeError("Unknown Error")
        result = get_json(result)
        if result['status']:
            self.username = username
            self.userapikey = result['userapikey']
            self.save()
        else:
            raise RuntimeError(result['error'])

    def login(self, username, password):
        ''' Log a user into a bokeh server.

        .. note::
            This is useful in multi-user mode.

        Args:
            username (str) : user name to log in
            password (str) : user password

        Returns:
            None

        '''
        url = urljoin(self.root_url, "bokeh/login")
        result = self.execute('post', url, data={
            'username': username,
            'password': password,
            'api': 'true'
        })
        if result.status_code != 200:
            raise RuntimeError("Unknown Error")
        result = get_json(result)
        if result['status']:
            self.username = username
            self.userapikey = result['userapikey']
            self.save()
        else:
            raise RuntimeError(result['error'])

        self.save()

    def browser_login(self):
        """ Open a browser with a token that logs the user into a bokeh server.

        .. note::
            This is useful in multi-user mode.

        Return:
            None

        """
        controller = browserlib.get_browser_controller()
        url = urljoin(self.root_url, "bokeh/loginfromapikey")
        url += "?" + urlencode({'username': self.username,
                                'userapikey': self.userapikey})
        controller.open(url)

    def data_source(self, name, data):
        """ Makes and uploads a server data source to the server.

        .. note::
            The server must be configured with a data directory.

        Args:
            name (str) : name for the data source object
            data (pd.DataFrame or np.array) : data to upload

        Returns:
            a ServerDataSource

        """
        raise NotImplementedError

    def list_data(self):
        """ Return all the data soruces on the server.

        Returns:
            sources : JSON

        """
        raise NotImplementedError

    def publish(self):
        url = urljoin(self.root_url, "/bokeh/%s/publish" % self.docid)
        self.post_json(url)

    def execute(self, method, url, headers=None, **kwargs):
        """ Execute an HTTP request using the current session.

        Returns the response

        Args:
            method (string) : 'get' or 'post'
            url (string) : url
            headers (dict, optional) : any extra HTTP headers

        Keyword Args:
            Any extra arguments to pass into the requests library

        Returns:
            response

        Returns the response
        """
        import requests
        import warnings
        func = getattr(self.http_session, method)
        try:
            resp = func(url, headers=headers, **kwargs)
        except requests.exceptions.ConnectionError as e:
            warnings.warn("You need to start the bokeh-server to see this example.")
            raise e
        if resp.status_code == 409:
            raise DataIntegrityException

        if resp.status_code == 401:
            raise Exception('HTTP Unauthorized accessing')
        return resp

    def execute_json(self, method, url, headers=None, **kwargs):
        """ same as execute, except ensure that json content-type is
        set in headers and interprets and returns the json response
        """
        if headers is None:
            headers = {}
        headers['content-type'] = 'application/json'
        resp = self.execute(method, url, headers=headers, **kwargs)
        return get_json(resp)

    def get_json(self, url, headers=None, **kwargs):
        """ Return the result of an HTTP 'get'.

        Args:
            url (str) : the URL for the 'get' request
            headers (dict, optional) : any extra HTTP headers

        Keyword Args:
            Any extra arguments to pass into the requests library

        Returns:
            response: JSON

        """
        return self.execute_json('get', url, headers=headers, **kwargs)

    def post_json(self, url, headers=None, **kwargs):
        """ Return the result of an HTTP 'post'

        Args:
            url (str) : the URL for the 'get' request
            headers (dict, optional) : any extra HTTP headers

        Keyword Args:
            Any extra arguments to pass into the requests library

        Returns:
            response: JSON

        """
        return self.execute_json('post', url, headers=headers, **kwargs)

    @property
    def userinfo(self):
        if not hasattr(self, "_userinfo"):
            url = urljoin(self.root_url, 'bokeh/userinfo/')
            self._userinfo = self.get_json(url)
        return self._userinfo

    @userinfo.setter
    def userinfo(self, val):
        self._userinfo = val

    @property
    def base_url(self):
        return urljoin(self.root_url, "bokeh/bb/")

    def get_api_key(self, docid):
        """ Retrieve the document API key from the server.

        Args:
            docid (string) : docid of the document to retrive API key for

        Returns:
            apikey : string

        """
        url = urljoin(self.root_url,"bokeh/getdocapikey/%s" % docid)
        apikey = self.get_json(url)
        if 'apikey' in apikey:
            apikey = apikey['apikey']
            logger.info('got read write apikey')
        else:
            apikey = apikey['readonlyapikey']
            logger.info('got read only apikey')
        return apikey

    def find_doc(self, name):
        """ Return the docid of the document with a title matching ``name``.

        .. note::
            Creates a new document with the given title if one is not found.

        Args:
            name (string) : name for the document

        Returns:
            docid : str

        """

        docs = self.userinfo.get('docs')

        matching = [x for x in docs if x.get('title') == name]

        if len(matching) == 0:
            logger.info("No documents found, creating new document '%s'" % name)
            self.make_doc(name)
            return self.find_doc(name)

        elif len(matching) > 1:
            logger.warning("Multiple documents with name '%s'" % name)

        return matching[0]['docid']

    def use_doc(self, name=None, docid=None):
        """ Configure the session to use a given document.

        Args:
            name (str, optional) : name of the document to use
            docid (str, optional) : id of the document to use

        .. note::
            only one of ``name`` or ``docid`` may be supplied.

        Creates a document for with the given name if one is not present on
        the server.

        Returns:
            None

        """
        if docid is not None and name is not None:
            raise ValueError("only one of 'name' or 'docid' can be supplied to use_doc(...)")

        if docid:
            self.docid = docid
        else:
            self.docid = self.find_doc(name)
        self.apikey = self.get_api_key(self.docid)

    def make_doc(self, title):
        """ Makes a new document with the given title on the server

        .. note:: user information is reloaded

        Returns:
            None

        """
        url = urljoin(self.root_url,"bokeh/doc/")
        data = protocol.serialize_json({'title' : title})
        self.userinfo = self.post_json(url, data=data)

    def pull(self, typename=None, objid=None):
        """ Pull JSON objects from the server.

        Returns a specific object if both ``typename`` and ``objid`` are
        supplied. Otherwise, returns all objects for the currently configured
        document.

        This is a low-level function.

        Args:
            typename (str, optional) : name of the type of object to pull
            objid (str, optional) : ID of the object to pull

        .. note::
            you must supply either ``typename`` AND ``objid`` or omit both.

        Returns:
            attrs : JSON

        """
        if typename is None and objid is None:
            url = urljoin(self.base_url, self.docid +"/")
            attrs = self.get_json(url)

        elif typename is None or objid is None:
            raise ValueError("typename and objid must both be None, or neither.")

        else:
            url = urljoin(
                self.base_url,
                self.docid + "/" + typename + "/" + objid + "/"
            )
            attr = self.get_json(url)
            attrs = [{
                'type': typename,
                'id': objid,
                'attributes': attr
            }]
        return attrs

    def push(self, *jsonobjs):
        """ Push JSON objects to the server.

        This is a low-level function.

        Args:
            *jsonobjs (JSON) : objects to push to the server

        Returns:
            None

        """
        data = protocol.serialize_json(jsonobjs)
        url = urljoin(self.base_url, self.docid + "/", "bulkupsert")
        self.post_json(url, data=data)

    def gc(self):
        url = urljoin(self.base_url, self.docid + "/", "gc")
        self.post_json(url)

    # convenience functions to use a session and store/fetch from server

    def load_document(self, doc):
        """ Loads data for the session and merge with the given document.

        Args:
            doc (Document) : document to load data into

        Returns:
            None

        """
        self.gc()
        json_objs = self.pull()
        doc.merge(json_objs)
        doc.docid = self.docid

    def load_object(self, obj, doc):
        """ Update an object in a document with data pulled from the server.

        Args:
            obj (PlotObject) : object to be updated
            doc (Document) : the object's document

        Returns:
            None

        """
        assert obj._id in doc._models
        attrs = self.pull(typename=obj.__view_model__, objid=obj._id)
        doc.load(*attrs)

    def store_document(self, doc, dirty_only=True):
        """ Store a document on the server.

        Returns the models that were actually pushed.

        Args:
            doc (Document) : the document to store
            dirty_only (bool, optional) : whether to store only dirty objects. (default: True)

        Returns:
            models : list[PlotObject]

        """
        doc._add_all()
        models = doc._models.values()

        if dirty_only:
            models = [x for x in models if getattr(x, '_dirty', False)]

        json_objs = doc.dump(*models)
        self.push(*json_objs)

        for model in models:
            model._dirty = False

        return models

    def store_objects(self, *objs, **kwargs):
        """ Store objects on the server

        Returns the objects that were actually stored.

        Args:
            *objs (PlotObject) : objects to store

        Keywords Args:
            dirty_only (bool, optional) : whether to store only dirty objects. (default: True)

        Returns:
            models : set[PlotObject]

        """

        models = set()

        for obj in objs:
            models.update(obj.references())

        if kwargs.pop('dirty_only', True):
            models = list(models)

        json_objs = dump(models, self.docid)
        self.push(*json_objs)

        for model in models:
            model._dirty = False

        return models

    def object_link(self, obj):
        """ Return a URL to a server page that will render the given object.

        Args:
            obj (PlotObject) : object to render

        Returns:
            URL string

        """
        link = "bokeh/doc/%s/%s" % (self.docid, obj._id)
        return urljoin(self.root_url, link)

    def show(self, obj):
        """ Display an object as HTML in IPython using its display protocol.

        Args:
            obj (PlotObject) : object to display

        Returns:
            None

        """
        data = {'text/html': autoload_server(obj, self)}
        publish_display_data(data)

    def poll_document(self, document, interval=0.5):
        """ Periodically ask the server for updates to the `document`. """
        try:
            while True:
                self.load_document(document)
                time.sleep(interval)
        except KeyboardInterrupt:
            print()
        except ConnectionError:
            print("Connection to bokeh-server was terminated")

    # helper methods

    def _prep_data_source_df(self, name, dataframe):
        name = tempfile.NamedTemporaryFile(prefix="bokeh_data",
                                           suffix=".pandas").name
        store = pd.HDFStore(name)
        store.append("__data__", dataframe, format="table", data_columns=True)
        store.close()
        return name

    def _prep_data_source_numpy(self, name, arr):
        name = tempfile.NamedTemporaryFile(prefix="bokeh_data",
                                           suffix=".table").name
        store = tables.File(name, 'w')
        store.createArray("/", "__data__", obj=arr)
        store.close()
        return name

class TestSession(Session):
    """Currently, register and login do not work, everything else should work
    in theory, but we'll have to test this as we go along and convert tests
    """
    def __init__(self, *args, **kwargs):
        if 'load_from_config' not in kwargs:
            kwargs['load_from_config'] = False
        self.client = kwargs.pop('client')
        self.headers = {}
        super(TestSession, self).__init__(*args, **kwargs)

    @property
    def username(self):
        return self.headers.get('BOKEHUSER')

    @username.setter
    def username(self, val):
        self.headers.update({'BOKEHUSER': val})

    @property
    def userapikey(self):
        return self.headers.get('BOKEHUSER-API-KEY')

    @userapikey.setter
    def userapikey(self, val):
        self.headers.update({'BOKEHUSER-API-KEY': val})

    def execute(self, method, url, headers=None, **kwargs):
        if headers is None:
            headers = {}
        func = getattr(self.client, method)
        resp = func(url, headers=headers, **kwargs)
        if resp.status_code == 409:
            raise DataIntegrityException
        if resp.status_code == 401:
            raise Exception('HTTP Unauthorized accessing')
        return resp
